#
#  @file   TurboGearsJRunnable 
#  @author 
#  date    
#  
#  Description: Python class implementing the JRunnable interface which 
#  setups and starts Turbogears. Inspired of start-<project_name>.py scripts 
#  generated by tg-admin quickstart and service.py of PythonService framework
#
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#
 

#JRunnable interface
# run_i()
# terminate()

import sys
import os
# Threading must be imported very early when the process is started, 
# otherwise we will run into the following exception
#  File
#"/usr/local/python2.5/lib/python2.5/threading.py", line
#540, in __delete
#    del _active[_get_ident()] 
# "More" explanations are given at: 
# http://bugs.python.org/issue1562822
import threading

import logging
from paraxip.infra.PythonLoggerForParaxip import PythonLoggerForParaxip

from paraxip.infra.PyLoggingFunctions import log_info, log_debug, log_error
import netborderPyInfra

#  Derive from PythonLoggerForParaxip to set the logger prefix
class myLoggerClass(PythonLoggerForParaxip):
  def __init__(self, in_strLoggerName, level=logging.NOTSET):
    PythonLoggerForParaxip.__init__(self,in_strLoggerName, level, 
                                    in_strPxipPrefix = "netborder.turboGears")
                                    
class TurboGearsJRunnable:

  def __init__(self):
    self._logger = netborderPyInfra.Logger(
        'netborder.oamgui.TurboGearsJRunnable')
    log_info(self._logger, 'TurboGearsJRunnable ctor')
    self._projectName = 'pstnconfig'      

  # JRunnable I/F
  def run_i(self):

    logging.setLoggerClass(myLoggerClass)

    # DO NOT IMPORT the commands module in ctor, toscawidgets uses thread-local 
    # storage, so it must be initialized in the same thread as it is used, see
    #   http://groups.google.com/group/toscawidgets-discuss/browse_thread/thread/44501869c52d81d7
    #
    # Exception:
    # 
    # TypeError: No object (name: ToscaWidgets per-request storage) has been 
    # registered for this thread

    # Set current directory to netborder.Installation.Directory (so TurboGears logs can be
    # sent to the proper location
    strInstallDir = netborderPyInfra.PyGlobalConfig.getParameter("netborder.Installation.Directory")
    if strInstallDir is None:
      log_error("netborder.Installation.OamWebServer.Path parameter not set. Cannot "\
                 "determine installation path. Ensure that "\
                 "netborder.Installation.OamWebServer.Path parameter is set.")       
      return
    else:
      os.chdir(strInstallDir)

    log_debug(self._logger, "Python syspath = %s" % sys.path)
    # Need a fromList to be set to a variable to the 'commands' submodule
    project_commands_module = \
      __import__(name='%s.commands' % self._projectName, fromlist='*')

    self._startFunction = project_commands_module.start

    # Hack: TurboGears project can get there config file name from 
    #       argument passed to Python. We are setting these argument 
    #       programaticaly   
    #  TODO: Read this value from config.
    sys.argv[0]="DummyScript"
    strProdConfigFileName = os.path.join(strInstallDir,"config")
    strProdConfigFileName = os.path.join(strProdConfigFileName,
                                         "oamWebServer.prod.cfg")
    sys.argv.append(strProdConfigFileName)
    # Disable auto-reload. (note: we perform cherrypy import only here 
    # because we don't want to have any threading issue by importing in the
    # main and performing action in this thread.
    import cherrypy
    cherrypy.lowercase_api = True
    cherrypy.config.update({'global': {'autoreload.on': False},})
    log_info(self._logger, 'TurboGearsJRunnable invoking start() command of '\
      'project "%s""' % self._projectName)

    self._startFunction() # Blocking

  def terminate(self):
    log_info(self._logger, 'TurboGearsJRunnable stopping '\
      'project "%s" ' % self._projectName)
    import cherrypy
    cherrypy.server.stop()

if __name__ == '__main__':
  t = TurboGearsJRunnable()
## !!MIRROR FILE, DO NOT MODIFY!!
