"""

File: TempDirJob.py
Author: Jonathan Maurice
Date: 14/02/2011
Description: Contain a set of job classes that are used to 
             call specific function of the TempDirManager

Copyright (C) 2003-2011 Sangoma Technologies.
All Rights Reserved.

"""

from paraxip.infra.PyLoggingFunctions import  *
from netborderPyInfra import TraceScope
import paraxip.infra.PyJob as PyJob

fsLogger = Logger("netborder.smartUpdate.TempDirJob")


class TempDirJob(PyJob.Job):
  """TempDirJob interface"""
  
  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "TempDirJob.__init__")
    super(TempDirJob, self).__init__()
    self.__tempDirManager = tempDirManager

  def run(self):
    pass


class TempDirInitialJob(TempDirJob):
  """Call TempDirManager initialConvert()"""

  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "TempDirInitialJob.__init__")
    super(TempDirInitialJob, self).__init__(tempDirManager)
    self.__tempDirManager = tempDirManager

  def run(self):
    t = TraceScope(fsLogger, "TempDirInitialJob.run")
    return self.__tempDirManager.initialConvert()


class SwapJob(TempDirJob):
  """Move missing files in destination directory, then
     swap directories. Afterward, make sure that
     destination directory is empty."""

  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "SwapJob.__init__")
    super(SwapJob, self).__init__(tempDirManager)
    self.__tempDirManager = tempDirManager
  
  def run(self):
    t = TraceScope(fsLogger, "SwapJob.run")

    if self.__tempDirManager.moveRemainingFiles() is False:
      return False
    if self.__tempDirManager.swapDirectories() is False:
      return False

    return self.__tempDirManager.eraseDstDirContent()


class SetTempModeJob(TempDirJob):
  """Call TempDirManager setTempMode()"""
  
  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "SetTempModeJob.__init__")
    super(SetTempModeJob, self).__init__(tempDirManager)
    self.__tempDirManager = tempDirManager
  
  def run(self):
    t = TraceScope(fsLogger, "SetTempModeJob.run")
    return self.__tempDirManager.setTempMode()


class SetFinalModeJob(TempDirJob):
  """Call TempDirManager setFinalMode()"""

  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "SetFinalModeJob.__init__")
    super(SetFinalModeJob, self).__init__(tempDirManager)
    self.__tempDirManager = tempDirManager
  
  def run(self):
    t = TraceScope(fsLogger, "SetFinalModeJob.run")
    return self.__tempDirManager.setFinalMode()


class CleanTempDirectoriesJob(TempDirJob):
  """Call TempDirManager cleanTempDirectories()"""

  def __init__(self, tempDirManager):
    t = TraceScope(fsLogger, "CleanTempDirectoriesJob.__init__")
    super(CleanTempDirectoriesJob, self).__init__(tempDirManager)
    self.__tempDirManager = tempDirManager
    
  def run(self):
    t = TraceScope(fsLogger, "CleanTempDirectoriesJob.run")
    return self.__tempDirManager.cleanTempDirectories()


## !!MIRROR FILE, DO NOT MODIFY!!
