<?xml version="1.0" encoding="utf-8"?>

<!--

This xslt stylesheet is used to generate python code base on the supplied
xml file. When invoked the stylesheet required an initVersion parameter
which is the initial version of the update process.

-->

<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method = "text"/> 


<xsl:template match="/">
  <xsl:call-template name="writeHeader"/>
  <xsl:call-template name="searchConverter">
    <xsl:with-param name="srcVersion" select="$initVersion"/>
  </xsl:call-template>
</xsl:template>


<!-- 
This template search for a converter element with an attribute
equal to the srcVersion parameter an apply other template to it.
If more then one converter element match or 0 return an error message.
-->
<xsl:template name="searchConverter">
  <xsl:param name="srcVersion"/>
  <xsl:if test="count(//smartUpdate/convert[@src=$srcVersion]) = 0">
    <xsl:message terminate="yes">
      <xsl:text>Invalid source version: Can't find a src attribute matching version </xsl:text>
      <xsl:value-of select="$srcVersion"/>
      <xsl:text> in smart update xml file</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:if test="count(//smartUpdate/convert[@src=$srcVersion]) &gt; 1">
    <xsl:message terminate="yes">
      <xsl:text>Invalid smart update xml file: Found more then one src attribute matching version </xsl:text>
      <xsl:value-of select="$srcVersion"/>
    </xsl:message>
  </xsl:if>
  <xsl:apply-templates select="//smartUpdate/convert[@src=$srcVersion]"/>
</xsl:template>


<!--
On a convert element, first check that it contain a job element.
If that's the case, add the swapJob to the job runner and apply template on job.
If @src = $initVersion, don't add swapJob to the job runner.
If @src = @dst, this means that this is the last convert element
(end of the update process). In that case, write the file footer.
Otherwise, call searchConverter template with srcVersion equal to @dst.
-->
<xsl:template match="convert">
  <xsl:if test="count(child::job) &gt; 0">
    <xsl:text>
    </xsl:text>
    <xsl:if test="@src!=$initVersion">
      <xsl:text>
  jobRunner.addJob(swapJob)</xsl:text>
    </xsl:if>
  </xsl:if>
  <xsl:choose>
    <xsl:when test="@src=@dst">
      <xsl:text>
  jobRunner.addJob(finalJob)</xsl:text>
      <xsl:apply-templates select="child::job"/>
      <xsl:call-template name="writeFooter"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="child::job"/>
      <xsl:call-template name="searchConverter">
        <xsl:with-param name="srcVersion" select="@dst"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
When a match is made on a job element the @converter
template must be applied first. Then the rest of the 
attributes are transformed to:
attribute_name="attribute_value", 
If it's the last attribute the function call is closed
with ") instead of ", like the others attributes.
-->
<xsl:template match="job">
  <xsl:apply-templates select="@converter"/>
  <xsl:for-each select="@*[not(name(.)='converter')]">
    <xsl:value-of select="name(.)"/>="<xsl:value-of select="."/>
    <xsl:if test="position()!=last()">
      <xsl:text>", </xsl:text>
    </xsl:if>
  </xsl:for-each>
  <xsl:text>", jobStatus=jobStatus))</xsl:text>
</xsl:template>


<!--
Added value of src attribute to the job constructor.
-->
<xsl:template match="@src">
  <xsl:text>"</xsl:text>
  <xsl:value-of select="."/>
  <xsl:text>", </xsl:text>
</xsl:template>


<!--
Added value of dst attribute to the job constructor.
-->
<xsl:template match="@dst">
  <xsl:text>"</xsl:text>
  <xsl:value-of select="."/>
  <xsl:text>", </xsl:text>
</xsl:template>


<!--
A converter attribute mean that a job corresponding to
the value of this attribute must be added to the job runner.
This template also apply template @src and @dst of it's parent
convert element.
-->
<xsl:template match="@converter">
  <xsl:text>
  jobRunner.addJob(Converter.</xsl:text>
  <xsl:value-of select="."/>
  <xsl:text>(tempDirManager, </xsl:text>
  <xsl:apply-templates select="ancestor::convert/@*"/>
</xsl:template>


<!--
Header of the .py generated.
-->
<xsl:template name="writeHeader">
  <xsl:text>

import paraxip.smart_update.Converter as Converter

def addJobs(jobRunner, jobStatus, tempDirManager, swapJob, finalJob):</xsl:text>
</xsl:template>

<!--
End of the .py generated.
-->
<xsl:template name="writeFooter">
  <xsl:text>
  
  return True</xsl:text>
</xsl:template>

</xsl:stylesheet>
