import logging

import turbogears
import os
import simplejson
import netborderPyThread
import urllib
import socket
from odict import odict
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from turbogears import config
from operator import itemgetter
from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN
from netborderPyInfra import PyGlobalConfig as globalConfig


log = logging.getLogger("pstnconfig.controllers")

channelStatusDict = {
  'INITIALIZING':'i',
  'INITIALIZING*':'i*',
  'ALARMED':'a',
  'ALARMED*':'a*',
  'IDLE':'I',
  'IDLE*':'I*',
  'ANSWERING':'A',
  'ANSWERING*':'A*',
  'CONNECTED':'C',
  'CONNECTED*':'C*',
  'DIALING':'D',
  'DIALING*':'D*',
  'RINGING':'R',
  'RINGING*':'R*',
  'HANGING-UP':'H',
  'HANGING-UP*':'H*',
  'GOING-OUT-OF-SERVICE':'G',
  'GOING-OUT-OF-SERVICE*':'G*',
  'OUT-OF-SERVICE':'O',
  'OUT-OF-SERVICE*':'O*',
  'OUT-OF-SERVICE(FINAL)':'F',
  'OUT-OF-SERVICE(FINAL)*':'F*'
}

# loggers to be listed in the web UI
loggerList = [
  'netborder',
  'netborder.ace',
  'netborder.oam',
  'netborder.stdsoap2',
  'netborder.appWeb',
  'netborder.sip.message',
  'netborder.voip',
  'netborder.gw',
  'netborder.ds',
  'netborder.media',
  'netborder.pstn',
  'netborder.net',
  'netborder.thread'
]

#
logPropertiesList = [
  'Mode',
  'Call Logger',
  'Root Logger',
  'SIP Logging',
  'PSTN Logging',
  'Media Logging'
]


_strServiceName = ""

class StatusController(controllers.Controller):
  def __init__(self, in_strProjectPath, in_webServiceObj):
    log.info("Initializing status controller")
    global strServiceName
    strServiceName = globalConfig.getParameter('netborder.oamWebServer.gwServiceName')

    if  strServiceName == None:
      log.error("Can't find name of gateway service in configuration. "
                 "Ensure that netborder.oamWebServer.gwServiceName is set "
                 "in configuration.")
      # Let's try Netborder Gateway ...
      strServiceName = "Netborder Gateway"
    log.debug("GW service name = %s",strServiceName)

    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                                         "javascript/widgets_src/status.js")

    self.webService = in_webServiceObj

    # Get the start button request timeout value
    startGwTimeout = \
      globalConfig.getParameter("netborder.oamWebServer.startGwTimeoutMs")

    #Elements that will build our tabber
    tabs = odict()
    #Each elements of the tabs dictionnary has the format:
    # tabs['Tab Text'] = ['/autoloadUrl','tabId',"Tooltip Text"]
    tabs['Controls'] = ['','controlsTabId',"Controls of the gateway as Starting and Stopping the gateway."]
    tabs['System Status'] = ['','systemStatusTabId',"Error report and call statistics"]
    tabs['Channel Status'] = ['','channelsTabId',"See channels statuses and allows to quiesce/put in service the channels"]
    #tabs['Alarms'] = ['','alarmsTabId',""]
    tabs['Logging'] = ['','loggingTabId',"Dynamic logging configuration"]

    items = [{'title': k, 'autoLoad': v[0], 'id': v[1], 'tabTip': v[2]} for k, v in tabs.items()]
    tabList = [(k.replace(" ","").lower(),v[0],v[1],v[2]) for k, v in tabs.items()]
    tabs = dict(tabs)

    self.statusSubTabs = TabPSTN( 'statusSubTabs',
                                  tabs=tabs,
                                  items=items,
                                  tabList=tabList,
                                  src_file=strSrcFile,
                                  startGwTimeout=startGwTimeout)


  def getSubTabs (self):
    return self.statusSubTabs

  @expose()
  def index (self):
    """Displays the status of the gateway using http get to localhost:7782/script/vital_sign.esp"""
    gwStarted = self.isGatewayStarted()
    return {}

  @expose(template='pstnconfig.templates.status.controls')
  def lightbox_controls(self, pingGW="false"):
    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    startGwTimeout = \
      globalConfig.getParameter("netborder.oamWebServer.startGwTimeoutMs")

    return \
    {
      "footerText":"",
      "isGatewayInCLI":isGatewayInCLI,
      "startGwTimeout":startGwTimeout,
      "pingGW":pingGW
    }

  @expose(template='pstnconfig.templates.status.system')
  def lightbox_system_status(self):
    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"
      
    if self.isGatewayReady():
      isGatewayReady = "true"
    else:
      isGatewayReady = "false"

    return \
    {
      "footerText":""
      ,"isGatewayInCLI":isGatewayInCLI
      ,"isGatewayReady":isGatewayReady
    }

  @expose(template='pstnconfig.templates.status.channels')
  def lightbox_channels_status(self):
    

    if self.isGatewayReady():
      isGatewayReady = "true"
    else:
      isGatewayReady = "false"
      
    if globalConfig.getParameter("netborder.oamWebServer.isGatewayInCLI"):
      isGatewayInCLI = "true"
    else:
      isGatewayInCLI = "false"

    return \
    {
      "footerText":""
      ,"isGatewayInCLI":isGatewayInCLI
      ,"isGatewayReady":isGatewayReady
    }

  @expose(template='pstnconfig.templates.status.logging')
  def lightbox_logging(self):

    return {"footerText":""}

  @expose()
  def restartGateway(self, _dc='None', js=''):
    self.stopGateway()
    if self.startGateway():
      return {'success':True}
    else:
      return {'success':False,'errorMsg':'Failed to start the gateway. See log file for details.'}

  @expose()
  def stopGatewayButton (self, _dc='None', js=''):
    return {"value":self.stopGateway()}

  def stopGateway(self):
    log.info("Stopping service")
    return netborderPyThread.stopService(strServiceName)

  @expose()
  def startGatewayButton (self, _dc='None', js=''):
    return {"value":self.startGateway()}

  def startGateway(self):
    log.info("nStarting service")
    return netborderPyThread.startService(strServiceName)

  # Use the getHostname method from the webService to test
  # wether the gateway is started or not. A socket error will
  # be raised if the gateway is not started.
  @expose()
  def pingGateway(self, _dc='None', js=''):
      return {'value':self.isGatewayReady()}
    
  # Use the getHostname method from the webService to test
  # wether the gateway is started or not. A socket error will
  # be raised if the gateway is not started.
  def isGatewayReady(self):
    try:
      alarmsStr = self.webService.getAlarms()
      log.debug("Alarms string: %s" % alarmsStr)
      isSystemReady = (alarmsStr.find("system.out-of-service : ALARM_ON") == -1)
      log.debug("Is system ready: %s" % isSystemReady)
      return isSystemReady
    except socket.error, e:
      return False

  @expose()
  def isGatewayServiceStarted(self, _dc='None', js=''):
    return {"value":self.isGWStarted()}

  def isGWStarted(self):
    return netborderPyThread.isServiceStarted(strServiceName)

  @expose()
  def changeGWStartupType(self, **data):
    log.info("Changing GW Startup type: %s" % data)
    result = changeGWStartupType(simplejson.loads(data['checkBoxValue']))
    return {'success':result}

  @expose()
  def changeGWFailureStartupType(self, **data):
    log.info("Changing GW Startup type: %s" % data)
    result = changeGWFailureStartupType(simplejson.loads(data['checkBoxValue']))
    return {'success':result}

  @expose()
  def isGWAutomaticStartup(self):
    return {"value":netborderPyThread.isServiceStartTypeAutomatic(strServiceName)}

  @expose()
  def isGWServiceFailureAutomaticRestartSet(self):
    return {"value":netborderPyThread.isServiceFailureAutomaticRestartSet(strServiceName)}

  ##########################################################################
  # Get the Error Level and the last error from the Gateway WebService
  @expose()
  def refreshErrors(self):
    errorData = []
    errorData.append({
      'errorDesc': 'Highest Error Level reached :',
      'errorValue': self.webService.getGWAlarmState()
    })
    errorData.append({
      'errorDesc': 'Last Error Received :',
      'errorValue': self.webService.getLastAlarm()
    })

    return(str(extjs.FormActionLoadResponse(success=True, data=errorData)))

  ##########################################################################
  # Get the calls statistics
  @expose()
  def getStatistics(self):
    stats = []
    stats.append({
      'statName': 'Total Calls',
      'currValue': self.webService.getNumStartedCall()
    })
    stats.append({
      'statName': 'Active Calls',
      'currValue': self.webService.getNumActiveCall()
    })
    stats.append({
      'statName': 'Max Active Calls',
      'currValue': self.webService.getMaxNumActiveCall()
    })
    stats.append({
      'statName': 'Average Call Duration(s)',
      'currValue': self.webService.getAvgCallDuration()
    })
    stats.append({
      'statName': 'Min Call Duration(s)',
      'currValue': self.webService.getMinCallDuration()
    })
    stats.append({
      'statName': 'Max Call Duration(s)',
      'currValue': self.webService.getMaxCallDuration()
    })
    return(str(extjs.FormActionLoadResponse(success=True, data=stats)))

  ##########################################################################
  # Resets the Error Level and the last error message
  @expose()
  def resetErrors(self, _dc='None', js=''):
    self.webService.resetAlarmState()
    return {}

  ##########################################################################
  # Put the channels out-of-service
  @expose()
  def quiesceChannels(self, _dc='None', js=''):
    self.webService.setChannelState('quiesce')
    return {}

  ##########################################################################
  # Bring the channels In Service
  @expose()
  def channelsInService(self, _dc='None', js=''):
    self.webService.setChannelState('is')
    return {}

  ##########################################################################
  # Get the channels informations
  # Received string is composed of lines with the following form:
  #  - b1(B1 - virtual)di1(B1I1)-c1 IDLE\n
  # Build a dictionnary composed of the headers of the grid and a list
  # of all the statuses. Returned JSON string has the form:
  # data = {
  #   'headers':[
  #     {'headerName':'01','dataIndex':'01'},
  #     {'headerName':'01','dataIndex':'01'},...
  #   ],
  #   'statusList':[
  #     {'displayName':'b1di1','boardName':'B1-Virtual','interfaceName':'B1I1','01':I,'02':I,...},
  #     {'displayName':'b1di1','boardName':'B1-Virtual','interfaceName':'B1I2','01':I,'02':I,...},...
  #   ]
  # }
  @expose()
  def getChannelStatus(self, firstCall, reorderAnalog = "false"):
    log.info("Getting Channels Status. Is First Call? %s" % firstCall)
    data = {}
    headerList = []
    statusList = []
    channelSplits = []
    analogChannelNum = {}
    
    # Get channels status from the GW
    chanList = self.webService.getChannelStatus()
    log.debug("Channels Status list: %s" % chanList)
    
    # Replace ( and ) signs with ; to ease the data splitting.
    chanList = chanList.replace("(",";")
    chanList = chanList.replace(")",";")
    # Split the received string on the line break sign to get
    # a list of all interface-channel pair.
    chanList = chanList.splitlines()
		
		# First loop to create an array where all analog inputs are listed on a 
		# single entry 

    
    for channel in chanList:
      # Once splitted in a list, all lines have the form:
      #   'b1;B1 - virtual;di1;B1I1;-c1 IDLE'
      # By splitting this string on the ; sign, we get the following list:
      #   ['b1', 'B1 - virtual', 'di1', 'B1I1', '-c1 IDLE']
      # For analog transform the folowing list:
      #   ['b2', 'B2 - virtual', 'ai1', 'B2I1', '-c1 IDLE']
      #   ['b2', 'B2 - virtual', 'ai2', 'B2I2', '-c1 IDLE']
      #   ['b2', 'B2 - virtual', 'ai3', 'B2I3', '-c1 IDLE']
      #   ['b2', 'B2 - virtual', 'ai4', 'B2I4', '-c1 IDLE']
      #   ['b3', 'B3 - virtual', 'ai1', 'B3I1', '-c1 IDLE']
      #   ['b3', 'B3 - virtual', 'ai2', 'B3I2', '-c1 IDLE']
      #   ['b3', 'B3 - virtual', 'ai3', 'B3I3', '-c1 IDLE']
      #   ['b3', 'B3 - virtual', 'ai4', 'B3I4', '-c1 IDLE']
      
      # into
      #   ['b2', 'B2 - virtual', 'ai2', 'B2I1', '-c1 IDLE']
      #   ['b2', 'B2 - virtual', 'ai2', 'B2I1', '-c2 IDLE']
      #   ['b2', 'B2 - virtual', 'ai2', 'B2I1', '-c3 IDLE']
      #   ['b2', 'B2 - virtual', 'ai2', 'B2I1', '-c4 IDLE']
      #   ['b3', 'B3 - virtual', 'ai3', 'B3I1', '-c1 IDLE']
      #   ['b3', 'B3 - virtual', 'ai3', 'B3I1', '-c2 IDLE']
      #   ['b3', 'B3 - virtual', 'ai3, 'B3I1', '-c3 IDLE']
      #   ['b3', 'B3 - virtual', 'ai3, 'B3I1', '-c4 IDLE']
      
      
      splitInfo         = channel.split(";")
      status            = (splitInfo[4].rpartition(" "))[2]
			
      # If it is an analog board
      if splitInfo[2].find("ai") >= 0 and reorderAnalog == "true":
        boardNum = splitInfo[0].partition("b")[2]
        boardNameSplit = splitInfo[3].partition("I")
        # If the board is encountered for the first time, initialize its
        # channel counter.
        if boardNum not in analogChannelNum:
          analogChannelNum[boardNum]=1
        splitInfo[2] = "a"
        splitInfo[3] = boardNameSplit[0] + boardNameSplit[1] 
        channelNum = analogChannelNum[boardNum]
        splitInfo[4] = "-c" +  str(channelNum) + " " + status
        analogChannelNum[boardNum] += 1
        
      channelSplits.append(splitInfo);

    for splitInfo in channelSplits:
      # Once splitted in a list, all lines have the form:
      #   'b1;B1 - virtual;di1;B1I1;-c1 IDLE'
      # By splitting this string on the ; sign, we get the following list:
      #   ['b1', 'B1 - virtual', 'di1', 'B1I1', '-c1 IDLE']
      displayName       = splitInfo[0] + splitInfo[2]
      boardDisplayName  = splitInfo[1]
      ifDisplayName     = splitInfo[3]
      splitInfo         = splitInfo[4].rpartition(" ")
      status            = splitInfo[2]
      status            = channelStatusDict[status]
      channel           = splitInfo[0].partition("-c")[2]
      
      
      if int(channel) < 10:
        channel = '0' + channel

      isHeaderInList = False
      for header in headerList:
        if header['headerName'] == channel:
          isHeaderInList = True
          break
      if not isHeaderInList:
        headerList.append({'headerName':channel,'dataIndex':channel})

      channelStatuses = None
      for elem in statusList:
        if elem['displayName'] == displayName:
          channelStatuses = elem
          break
      if channelStatuses:
        channelStatuses[channel] = status
      else:
        statusList.append({'displayName':displayName,channel:status,'boardName':boardDisplayName,'interfaceName':ifDisplayName})

    headerList = sorted(headerList,key=itemgetter('headerName'))
    headerList.insert(0,{'headerName':'','dataIndex':'displayName'})
    # sort is done on display name only since maximum interface number is 8.
    # if interface number grows at 10 and more, need to do a numeric sort.
    statusList = sorted(statusList,key=itemgetter('displayName'))
    #statusList.reverse()

    if simplejson.loads(firstCall):
      data['headers'] = headerList
      data['statusList'] = statusList
    else:
      data = statusList
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

############################################################################
  # Get LogLevel of all the loggers
  # The failed result of the getLoglevel function is
  # {'m-strStatusText': 'Invalid logger : netborde22r', 'm-bExecSuccess': 'false'}
  # The sucessfull result of the getLogLevel function is
  # {'m-strStatusText': 'Success\nThe loglevel of netborder is INFO', 'm-bExecSuccess': 'true'}
  # The returned log level is indicated at the end of the m-strStatusText
  @expose()
  def getLogLevel(self):
    data = []
    for logger in loggerList:
      result = self.webService.getLogLevel(logger)
      log.debug("Result: %s", result)
      if (result['m-bExecSuccess'].find('true') == 0):
        data.append({
            'logger': logger,
            'level': result['m-strStatusText'].split(" is ")[1]
            })

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

############################################################################
  # Set LogLevel of a logger
  # records is a array with all the loggers to modify.
  # the format is [[level, logger1],[level, logger2]
  # we need to append the string "_LOG_LEVEL" to the incoming level
  # befor passing the level value to the webservice function
  @expose()
  def setLogLevel(self, records):
    log.debug("setloglevel Records list: %s", records)
    data = []
    for rec in simplejson.loads(records):
      level = rec[0] + "_LOG_LEVEL"
      logger = rec[1]
      log.debug("set-log-level %s %s", level, logger)
      result = self.webService.setLogLevel(level, logger)
      log.debug("Result: %s", result)
    return(str(extjs.FormActionLoadResponse(success=True,data=data)))


############################################################################
  def isLoggerListInDevelopment(self,in_loggerList):
    for logger in in_loggerList:
      result = self.webService.getLogLevel(logger)
      if (result['m-bExecSuccess'].find('true') == 0):
        level = result['m-strStatusText'].split(" is ")[1]
        if(level == 'WARN' or level == 'ERROR' or level == 'FATAL'):
          return False
    return True


############################################################################
  @expose()
  def getGlobalLogConfigProperties(self):
    log.debug("getGlobalLogConfigProperties")
    data = []

    for logproperty in logPropertiesList:
      result = None
      if(logproperty == 'Mode'):
        log.debug("get-run-mode")
        result = self.webService.getRunMode()
        log.debug("Result: %s", result)
      elif(logproperty == 'Call Logger'):
        log.debug("get-call-log-status")
        result = self.webService.getCallLoggerStatus()
        log.debug("Result: %s", result)
      elif(logproperty == 'Root Logger'):
         result = self.webService.getLogLevel('root')
      else:
        List = None
        if(logproperty == 'SIP Logging'):
          List = ['netborder.voip','netborder.sip.message']
        elif(logproperty == 'PSTN Logging'):
          List = ['netborder.pstn.sangoma.isdn.message','netborder.sangoma.isdn.channel',
                  'netborder.pstn.telesoft.analog.fxo.lld','netborder.pstn.sangoma.fxo.channel',
                  'netborder.pstn.telesoft.analog.fxs.lld','netborder.pstn.sangoma.fxs.channel']
        elif(logproperty == 'Media Logging'):
          List = ['netborder.media']
        else:
          List = None

        if(List is not None and self.isLoggerListInDevelopment(List) == True):
          result = {
            'm-bExecSuccess': 'true',
            'm-strStatusText': ' is debug'
            }
        else:
          result = {
            'm-bExecSuccess': 'true',
            'm-strStatusText': ' is normal'
            }

      if ( result is not None and result['m-bExecSuccess'].find('true') == 0):
        value = result['m-strStatusText'].split(" is ")[1]
        if(logproperty != 'Root Logger'):
          value = value.swapcase()
        data.append({
            'property': logproperty,
            'value': value
            })
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

############################################################################
  def updateLoggers(self,in_loggerList):
    for entry in in_loggerList:
      values = entry.split(";")
      logger = values[0]
      level  = values[1]
      log.debug("set-log-level %s %s", level, logger)
      result = self.webService.setLogLevel(level, logger)
      log.debug("Result: %s", result)

############################################################################
  # Set the Basic logger properties
  @expose()
  def setGlobalLogConfigProperties(self, records):
    log.debug("setGlobalLogConfigProperties Records list: %s", records)
    data = []

    for rec in simplejson.loads(records):
      logproperty = rec[0]
      if(logproperty == 'Mode'):
        runMode = rec[1].swapcase()
        log.debug("set-run-mode %s", runMode)
        result = self.webService.setRunMode(runMode)
        log.debug("Result: %s", result)
      elif(logproperty == 'Call Logger'):
        callLogStatus = rec[1].swapcase()
        log.debug("set-call-log-status %s", callLogStatus)
        result = self.webService.setCallLoggerStatus(callLogStatus)
        log.debug("Result: %s", result)
      else:
        List = None
        if(logproperty == 'Root Logger'):
          strCmd = 'root;' + rec[1]+'_LOG_LEVEL'
          List = [strCmd]

        elif(logproperty == 'SIP Logging'):
          if(rec[1] == 'DEBUG'):
            List = ['netborder.voip;DEBUG_LOG_LEVEL',
                    'netborder.sip.message;INFO_LOG_LEVEL']
          else:
            List = ['netborder.voip;WARN_LOG_LEVEL',
                    'netborder.sip.message;WARN_LOG_LEVEL']

        elif(logproperty == 'PSTN Logging'):
          if(rec[1] == 'DEBUG'):
            List = ['netborder.pstn.sangoma.isdn.message;INFO_LOG_LEVEL','netborder.sangoma.isdn.channel;DEBUG_LOG_LEVEL',
                    'netborder.pstn.telesoft.analog.fxo.lld;DEBUG_LOG_LEVEL','netborder.pstn.sangoma.fxo.channel;DEBUG_LOG_LEVEL',
                    'netborder.pstn.telesoft.analog.fxs.lld;DEBUG_LOG_LEVEL','netborder.pstn.sangoma.fxs.channel;DEBUG_LOG_LEVEL']
          else:
            List = ['netborder.pstn.sangoma.isdn.message;WARN_LOG_LEVEL','netborder.sangoma.isdn.channel;WARN_LOG_LEVEL',
                    'netborder.pstn.telesoft.analog.fxo.lld;WARN_LOG_LEVEL','netborder.pstn.sangoma.fxo.channel;WARN_LOG_LEVEL',
                    'netborder.pstn.telesoft.analog.fxs.lld;WARN_LOG_LEVEL','netborder.pstn.sangoma.fxs.channel;WARN_LOG_LEVEL']

        elif(logproperty == 'Media Logging'):
           if(rec[1] == 'DEBUG'):
             List = ['netborder.media;INFO_LOG_LEVEL']
           else:
             List = ['netborder.media;WARN_LOG_LEVEL']

        else:
          List = None

        if(List is not None):
          self.updateLoggers(List)

    return(str(extjs.FormActionLoadResponse(success=True,data=data)))

def changeGWStartupType(checkBoxValue):
  log.info("Changing GW Startup type: %s" % checkBoxValue)
  if checkBoxValue:
    result = netborderPyThread.setServiceAutomaticStartup(strServiceName)
  else:
    result = netborderPyThread.setServiceManualStartup(strServiceName)

  return {'success':result}

def changeGWFailureStartupType(checkBoxValue):
  log.info("Changing GW Failure Startup type: %s" % checkBoxValue)
  if checkBoxValue:
    result = netborderPyThread.setServiceFailureAutomaticRestartCount(strServiceName, 2)
  else:
    result = netborderPyThread.setServiceFailureAutomaticRestartCount(strServiceName, 0)

  return {'success':result}
