//------------------------------------------------------------
//  This file includes all the objects needed by the editor   |
//  to display the informations on the boards. Also includes  |
//  the method used to save the informations changed by the   |
//  user.                                                     |
//------------------------------------------------------------
Ext.namespace("Paraxip.config.boards");

//-----------------------------------------------
// Modify boards and interfaces methods
//-----------------------------------------------
var saveBoard = function(formButton){
  var form = formButton.ownerCt.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "boards/saveBoards";
  el.dom.method = 'POST';
  var boardInfo = formButton.ownerCt.getItemsValueAsJson();
  form.submit({
    params:{
      'boardInfo': boardInfo
    },
    success: function(){
       Paraxip.isConfigOutOfSynch(true);
       reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var saveInterface = function(formButton){
  var form = formButton.ownerCt.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "boards/saveInterfaces";
  el.dom.method = 'POST';

  // getItemsValueAsJson is a Ext.Container override
  var interfaceInfo = formButton.ownerCt.getItemsValueAsJson();
  form.submit({
    params:{
      'interfaceInfo': interfaceInfo
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var boardCancelFunc = function(){
  reloadTree(true);
};

var formCancel = function (form){
  form.scope.hide();
};

//-----------------------------------------------
// Add/Delete Interfaces methods
//-----------------------------------------------
var addInterfaceHandler = function(){

  addInterfaceWindow.show();
  if( this.ownerCt.id == "gridInterfacesAnalog" ) {
    boardType.setValue("analog");
    interfaceType.setContainerVisible(true);
    interfaceType.enable();
    if( interfaceType.value == "FXS") {
      phoneNumber.setContainerVisible(true);
      phoneNumber.enable();
      callerName.setContainerVisible(true);
      callerName.enable();
    }
  } else if( this.ownerCt.id == "gridInterfacesDigital" ) {
    boardType.setValue("digital");
    interfaceType.setContainerVisible(false);
    interfaceType.disable();
    phoneNumber.setContainerVisible(false);
    phoneNumber.disable();
    callerName.setContainerVisible(false);
    callerName.disable();
  }
};

var deleteInterfaceHandler = function(){
  var currentGrid = this.ownerCt;
  var selectedRecords = currentGrid.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    var boardID = 0;

    if( this.ownerCt.id == "gridInterfacesAnalog" ) {
      boardID = Ext.getCmp("Paraxip.config.boards.analogBoardID").value;
    } else if( this.ownerCt.id == "gridInterfacesDigital") {
      boardID = Ext.getCmp("Paraxip.config.boards.digitalBoardID").value;
    }

    var ifArray = new Array();
    for (var i = 0; i < selectedRecords.length; i++) {
      ifArray.push(selectedRecords[i].data);
    }
    ifArray = Ext.util.JSON.encode(ifArray);
    Ext.Ajax.request({
      url: Paraxip.baseUrl + "boards/deleteInterfaceFromEditor",
      method: 'POST',
      disableCaching: false,
      params: {
        "interfaces":ifArray,
        "boardID": Ext.util.JSON.encode(boardID)
      },
      success: function(){
        Paraxip.isConfigOutOfSynch(true);
        for (var index = 0; index < selectedRecords.length; index++) {
         currentGrid.store.remove(selectedRecords[index]);
        }
        reloadTree(true);
      },
      failure: Paraxip.failureMessage
    });
  }
};

var addInterface = function(){
  //submit information
  var form = addInterfaceForm.getForm();
  var el = form.getEl();
  var boardID = 0;

  if( boardType.getValue() == "analog" ) {
    boardID = Ext.getCmp("Paraxip.config.boards.analogBoardID").value;
  } else if( boardType.getValue() == "digital" ) {
    boardID = Ext.getCmp("Paraxip.config.boards.digitalBoardID").value;
  }

  var interfaceData = this.ownerCt.getItemsValueAsJson();

  el.dom.action = Paraxip.baseUrl + "boards/addInterfaceFromEditor";
  el.dom.method = 'POST';
  form.submit({
    params:{
      'interfaceData': interfaceData,
      'boardID': Ext.util.JSON.encode(boardID)
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

  addInterfaceWindow.hide();
};

//-----------------------------------------------
// Intefaces Grids
//-----------------------------------------------
var gridInterfacesDigital = new Ext.grid.GridPanel({
  title       : 'Interfaces Grid',
  id          : 'gridInterfacesDigital',
  autoScroll  : true,
  layout      : 'fit',
  store       : new Ext.data.JsonStore({
    id: 'gridInterfacesDigital',
    url: Paraxip.baseUrl + "boards/getGridDigitalInterfacesData",
    root: 'data',
    //,autoLoad: true
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'wanpipe'},
      {name: 'nodeId'} //Not displayed
    ]
  }),
  columns    :[
    {
      header: "ID",
      width: 35,
      sortable: true,
      dataIndex: 'id',
      css:'padding:5px;'
    },
    {
      header: "Name",
      width: 200,
      sortable: true,
      dataIndex: 'displayName',
      css:'padding:5px;'
    },
    {
      header: "Wanpipe",
      width: 100,
      sortable: true,
      dataIndex: 'wanpipe',
      css:'padding:5px;'
    }
  ],
  viewConfig :{forceFit:true}
}); // eo apply

if( hardwareDetectionMode ) {
// TODO - Add this button when ready to handle it.
//  gridInterfacesDigital.addButton("Add/Remove Interface(s)", addInterfaceHandler);
} else {
  gridInterfacesDigital.addButton("Add Interface", addInterfaceHandler);
  gridInterfacesDigital.addButton("Remove Interface(s)", deleteInterfaceHandler);
}

gridInterfacesDigital.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

var gridInterfacesAnalog = new Ext.grid.GridPanel({
  title       : 'Interfaces Grid',
  id          : 'gridInterfacesAnalog',
  autoScroll  : true,
  layout      : 'form',
  store       : new Ext.data.JsonStore({
    id: 'gridInterfacesAnalog',
    url: Paraxip.baseUrl + "boards/getGridAnalogInterfacesData",
    root: 'data',
    //,autoLoad: true
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'wanpipeInterface'},
      {name: 'nodeId'} //Not displayed
    ]
  }),
  columns    :[
    {
      header: "ID",
      width: 35,
      sortable: true,
      dataIndex: 'id',
      css:'padding:5px;'
    },
    {
      header: "Name",
      width: 200,
      sortable: true,
      dataIndex: 'displayName',
      css:'padding:5px;'
    },
    {
      header: "WanpipeInterface",
      width: 100,
      sortable: true,
      dataIndex: 'wanpipeInterface',
      css:'padding:5px;'
    }
  ],
  viewConfig :{forceFit:true}
}); // eo apply

gridInterfacesAnalog.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

if( hardwareDetectionMode ) {
// TODO - Add this button when ready to handle it.
//  gridInterfacesAnalog.addButton("Add/Remove Interface(s)", addInterfaceHandler);
} else {
  gridInterfacesAnalog.addButton("Add Interface", addInterfaceHandler);
  gridInterfacesAnalog.addButton("Remove Interface(s)", deleteInterfaceHandler);
}

var selectedFXS = function(){
  if( this.value == "FXS" ) {
    phoneNumber.setContainerVisible(true);
    phoneNumber.enable();
    callerName.setContainerVisible(true);
    callerName.enable();
  } else {
    phoneNumber.setContainerVisible(false);
    phoneNumber.disable();
    callerName.setContainerVisible(false);
    callerName.disable();
  }
};

var interfaceType = new Ext.form.ComboBox({
  id              : "interfaceType"
  ,fieldLabel     : "Type"
  ,name           : "type"
  ,store          : ["FXO","FXS"]
  ,forceSelection : true
  ,triggerAction  : 'all'
  ,mode           : 'local'
  ,typeAhead      : true
  ,value          : "FXO"
  ,selectOnFocus  : true
  ,width          : 130
  ,listWidth      : 155
});

interfaceType.on("select",selectedFXS,interfaceType);

var phoneNumber = new Ext.form.TextField({
  id: "phoneNumber"
  ,fieldLabel: "Phone Number"
  ,name: "phoneNumber"
  ,value: "phoneNumber1"
  ,width: 130
});

phoneNumber.on("render",function(){
  phoneNumber.setContainerVisible(false);
  phoneNumber.disable();
});

var callerName = new Ext.form.TextField({
  id: "callerName"
  ,name: "callerName"
  ,fieldLabel: "Caller Name"
  ,value: "user1"
  ,width: 130
});

callerName.on("render",function(){
  callerName.setContainerVisible(false);
  callerName.disable();
});

var boardType = new Ext.form.TextField({
  id: "boardType"
  ,disabled: true
  ,hidden: true
  ,hideLabel: true
  ,value: "digital"
});

var addInterfaceForm = new Ext.form.FormPanel({
  id          : "add_interface_form"
  ,region     : "center"
  ,bodyStyle  : 'padding:10px;'
  ,items      : [{
    name:"displayName"
    ,xtype:"textfield"
    ,id:"interfaceName"
    ,fieldLabel:"Interface Name"
  }]
});

addInterfaceForm.add(boardType);
addInterfaceForm.add(interfaceType);
addInterfaceForm.add(phoneNumber);
addInterfaceForm.add(callerName);

var addInterfaceWindow = new Ext.Window({
  title       : 'Add Interface',
  layout      : 'border',
  width       : 505,
  height      : 200,
  closeAction : 'hide',
  plain       : true,
  id          : "add_interface_window"
});

addInterfaceWindow.add( addInterfaceForm );
addInterfaceForm.addButton("Add", addInterface);
addInterfaceForm.addButton("Cancel", formCancel, addInterfaceWindow);


//-----------------------------------------------
// Digital Interfaces Details
//-----------------------------------------------
var frmDigitalInterface = new Ext.FormPanel({
  id          : 'frmDigitalInterface',
  title       : 'Interface Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  autoScroll  : true,
  labelPad    : 20,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',

  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.boards.digitalInterfaceID",
    fieldLabel: 'ID',
    name: 'id',
    helpText: "sangomaInterfaceCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.boards.digitalBriModType",
    fieldLabel: 'BRI Mode Type',
    name: 'briModType',
    disabled: true,
    helpText: "sangomaDigitalInterfacesinterfacebriModType"
  },{
    id: "Paraxip.config.boards.digitalInterfaceDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText: "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.boards.digitalInterfaceWanpipe",
    xtype: 'numberfield',
    fieldLabel: 'Wanpipe',
    name: 'wanpipe',
    helpText: "sangomaDigitalInterfacesinterfacewanpipe"
  }],

  buttons     : [{
    id: "Paraxip.config.boards.digitalInterfaceSaveButton",
    text: 'Save',
    handler: saveInterface
  },{
    id: "Paraxip.config.boards.digitalInterfaceCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

Paraxip.config.boards.changeChannelGroupID = function(e, newValue, oldValue) {
  if( Ext.getCmp("Paraxip.config.boards.analogInterfaceType").getValue() == "FXS" ) {
    Ext.Ajax.request({
      url: Paraxip.baseUrl + "boards/changeChannelGroupID"
      ,method: 'POST'
      ,disableCaching: false
      ,params: {"newID":newValue,"oldID":oldValue}
      ,success: function (result) {
        response = Ext.util.JSON.decode(result.responseText);
        if( !response.success ) {
          Ext.getCmp("Paraxip.config.boards.analogPhoneNumber").setValue(oldValue);
          Ext.Msg.alert(
            "Unable to change Channel Group ID",
            "Reverting to previous phone number because we were unable to " +
            "change the channel group ID. See Gateway Manager log file for " +
            "more details."
          );
        }
      }
      ,failure: Paraxip.failureMessage
    });
  }
}

var frmAnalogInterface = new Ext.FormPanel({
  id            : 'frmAnalogInterface'
  ,title        : 'Interface Details'
  ,labelWidth   : 75 // label settings here cascade unless overridden
  ,frame        : true
  ,bodyStyle    : 'padding:5px 5px 0'
  ,width        : 350
  ,autoScroll   : true
  ,labelPad     : 20
  ,layout       : 'form'
  ,layoutConfig : {
    labelSeparator: ''
  }
  ,defaults     : {
    width: 230
    ,msgTarget: 'side'
  }
  ,defaultType  : 'textfield'

  ,items        : [{
    name: 'nodeId'
    ,hidden: true
    ,hideLabel: true
  },{
    id: "Paraxip.config.boards.analogInterfaceID"
    ,fieldLabel: 'ID'
    ,name: 'id'
    ,helpText : "sangomaInterfaceCommonAttributesID"
    ,disabled: true
  },{
    id: "Paraxip.config.boards.analogInterfaceDisplayName"
    ,fieldLabel: 'Name'
    ,name: 'displayName'
    ,helpText : "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.boards.analogInterfaceWanpipeInterface"
    ,xtype: 'numberfield'
    ,fieldLabel: 'Wanpipe Interface'
    ,name: 'wanpipeInterface'
    ,helpText:"sangomaAnalogInterfacesinterfacewanpipeInterface"
  },{
    id: "Paraxip.config.boards.analogInterfaceType"
    ,xtype: "textfield"
    ,fieldLabel: "Interface Type"
    ,name: "interfaceType"
    ,disabled: true
    ,helpText: "sangomaAnalogInterfacesinterfacetype"
  },{
    id: "Paraxip.config.boards.analogPhoneNumber"
    ,fieldLabel: "Phone Number"
    ,name: "phoneNumber"
    ,helpText: "sangomaAnalogInterfacesinterfacephoneNumber"
    ,listeners: {
      'change' : Paraxip.config.boards.changeChannelGroupID
    }
  },{
    id          : "Paraxip.config.boards.analogCallerName"
    ,fieldLabel : "Caller Name"
    ,name       : "callerName"
    ,helpText   : "sangomaAnalogFxsInterfacesAttributescallerName"
  }]
  ,buttons      : [{
    id: "Paraxip.config.boards.analogInterfaceSaveButton"
    ,text: 'Save'
    ,handler: saveInterface
  },{
    id: "Paraxip.config.boards.analogInterfaceCancelButton"
    ,text: 'Cancel'
    ,handler: boardCancelFunc
  }]
});

Paraxip.config.boards.showFxsAttributes = function(){
  var analogInterfaceType =
    Ext.getCmp("Paraxip.config.boards.analogInterfaceType");

  if( analogInterfaceType.getValue() == "FXS" ){
    Ext.getCmp("Paraxip.config.boards.analogCallerName").setContainerVisible(true);
  } else {
    Ext.getCmp("Paraxip.config.boards.analogCallerName").setContainerVisible(false);
  }
};

var frmVirtualDigitalBoard = new Ext.FormPanel({
  id          : 'frmVirtualDigitalBoard',
  title       : 'Boards Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  labelPad    : 20,
  autoScroll  : true,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',
  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.boards.virtualBoardID",
    fieldLabel: 'ID',
    name: 'id',
    helpText : "sangomaInterfaceCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.boards.virtualBoardDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText : "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.boards.virtualBoardType",
    fieldLabel: 'Type',
    name: 'type',
    disabled: true
  },{
    id: "Paraxip.config.boards.virtualBoardModel",
    fieldLabel: 'Model',
    name: 'model',
    disabled: true
  },{
    id: "Paraxip.config.boards.virtualNumInterfaces",
    fieldLabel: 'Number of Interfaces',
    name: 'numInterfaces',
    disabled: true
  }],
  buttons     : [{
    id: "Paraxip.config.boards.virtualBoardSaveButton",
    text: 'Save',
    handler: saveBoard
  },{
    id: "Paraxip.config.boards.virtualBoardCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var panelVirtualDigitalBoard = new Ext.Panel({
  id           : 'panelVirtualDigitalBoard',
  region       : 'center',
  layout       : 'accordion',
  border       : false,
  closable     : false,
  buttonAlign  : 'right'
});

panelVirtualDigitalBoard.add(frmVirtualDigitalBoard);

var frmVirtualDigitalInterface = new Ext.FormPanel({
  id          : 'frmVirtualDigitalInterface',
  title       : 'Virtual Interface Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  autoScroll  : true,
  labelPad    : 20,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',

  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.boards.virtualInterfaceID",
    fieldLabel: 'ID',
    name: 'id',
    disabled: true
  },{
    id: "Paraxip.config.boards.virtualInterfaceDisplayName",
    fieldLabel: 'Name',
    name: 'displayName'
  },{
    id: "Paraxip.config.boards.peerVirtualInterfaceID",
    fieldLabel: 'Peer Virtual Interface ID',
    name: 'peerVirtualInterfaceID',
    helpText: "sangomaVirtualDigitalInterfacesinterfacepeerVirtualInterfaceID",
    xtype: 'numberfield',
    allowNegative: false,
    allowDecimals: false
  }],

  buttons     : [{
    id: "Paraxip.config.boards.virtualInterfaceSaveButton",
    text: 'Save',
    handler: saveInterface
  },{
    id: "Paraxip.config.boards.virtualInterfaceCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

//-----------------------------------------------
// Add/Delete Boards methods
//-----------------------------------------------
// recursive function that will load the interfaces from the ajax callback
var loadInterfacesForBoard = function(record)
{
  //Paraxip.console.debug("loadInterfacesForBoard: " + record);
  // As the function goes backward so the first record is loaded last, we cannot
  // select the interfaces of the last as this is the first call and it is not loaded
  if ((record != availableBoardsGrid.getStore().getRange().length -1) &&
      (availableBoardsGrid.getStore().getRange()[record+1].data.active))
  {
     Paraxip.selectInterfaces(availableInterfacesGrid);
  }
  // stop recursion
  if (record == -1) {
    // Select board.
    var boards = availableBoardsGrid.getStore().getRange();
    for( var i = 0; i < boards.length; ++i ) {
      if( boards[i].data.active ) {
        availableBoardsGrid.getSelectionModel().selectRow(i, true);
      }
    }
    return;
  }
  var dataRecord = availableBoardsGrid.getStore().getRange()[record];
  if (dataRecord.data.active) {

    // Update the interfaces grid headers according to the board model
    Paraxip.updateInterfacesGridHeaders(availableInterfacesGrid, dataRecord);

    // Only load if not already loaded.
    if( Paraxip.config.boards.interfacesStoreMap.get(dataRecord.data.id) != null ) {
      Paraxip.config.boards.interfacesStoreMap.remove(dataRecord.data.id);
    }
    availableInterfacesGrid.getStore().load({
      params: {"boardID":dataRecord.data.id}
      ,callback: function() {
        // callback is a recursion on previous record
        // change the title of the interfaces grid
        availableInterfacesGrid.setTitle(
          "Select Interfaces for Board " + dataRecord.data.id);
        loadInterfacesForBoard(--record);
      }
    });
    return;
  }

  // If not attempted to load go to previous record by recursion.
  loadInterfacesForBoard(--record);
}

var addBoardHandler = function(){

  if( hardwareDetectionMode ) {
    Paraxip.config.boards.interfacesStoreMap.clear();
    interfacesStore.removeAll();
    // Load the available boards then for each boards, load their interfaces
    boardsStore.load({
      params: {'js':'force_post_str'}
      ,callback: function(records) {
        loadInterfacesForBoard(--(records.length));
      }
    });
  }
  addBoardWindow.show(gridBoards);
};

var deleteBoardHandler = function(){
  var selectedRecords = gridBoards.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    var boards = new Array();
    for (var i = 0; i < selectedRecords.length; i++) {
      boards.push(selectedRecords[i].data);
    }
    boards = Ext.util.JSON.encode(boards);

    Ext.Ajax.request({
      url: Paraxip.baseUrl + "boards/deleteBoardFromEditor",
      method: 'POST',
      disableCaching: false,
      params: {"boards":boards},
      success: function(){
        Paraxip.isConfigOutOfSynch(true);
        reloadTree(true);
        if(Ext.getCmp('fxoTuneTabId') != undefined){
          Paraxip.config.fxoTune.update();
        }
      },
      failure: Paraxip.failureMessage
    });
  }
};

Paraxip.config.boards.hasFxsInterfaces = function(){
  var isAnalog = Paraxip.isBoardAnalog(boardSelection.getValue());
  var isBRI = Paraxip.isBoardBRI(boardSelection.getValue());
  if( isAnalog ) {
    interfacesNum = parseInt(fxoInterfacesNumber.getRawValue(), 10);
    interfacesNum += parseInt(fxsInterfacesNumber.getRawValue(), 10);

    if( interfacesNum < Paraxip.minAnalogInterfaces
        || interfacesNum > Paraxip.maxAnalogInterfaces ) {
      Ext.Msg.alert("Wrong number of analog interfaces",
                    "The total number of analog interfaces (fxo interfaces " +
                    "+ fxs interfaces) must be at least " +
                    Paraxip.minAnalogInterfaces + " interfaces and not more " +
                    "than " + Paraxip.maxAnalogInterfaces + " interfaces.");
      return;
    }
  }

  if( isAnalog && interfacesNum > 0 ) {
    addBoardWindow.hide();
    Paraxip.config.boards.addPhoneNumberFields();
    Paraxip.config.boards.editFxsWindow.show();
  } else if(isBRI){
    interfacesNum = parseInt(briInterfacesNumber.getRawValue(), 10);
    if( interfacesNum < Paraxip.minBriInterfaces
        || interfacesNum > Paraxip.maxBriInterfaces ) {
      Ext.Msg.alert("Wrong number of BRI interfaces",
        "The total number of BRI interfaces must be at least " +
        Paraxip.minBriInterfaces + " interfaces and not more " +
        "than " + Paraxip.maxBriInterfaces + " interfaces.");
      return;
    }
    addBoardWindow.hide();
    Paraxip.config.boards.addBriModTypeFields();
    Paraxip.config.boards.editBriWindow.show();
  } else {
    addBoard();
  }
};

Paraxip.config.boards.addPhoneNumberFields = function() {
  var i = 1;

  Paraxip.config.boards.fxsAttStore.removeAll();

  var numIfs = parseInt(fxsInterfacesNumber.getRawValue(), 10) +
    parseInt(fxoInterfacesNumber.getRawValue(), 10);

  for( i ; i <= numIfs; ++i ) {
    if( i <= parseInt(fxoInterfacesNumber.getRawValue(), 10) ) {
      var callerName = "N/A - FXO interface";
    } else {
      var callerName = "user" + i;
    }
    Paraxip.config.boards.fxsAttStore.add(
      new Paraxip.config.boards.FxsAttDataRecord({
        'interface_id':i
        ,'phone_number': "phoneNumber" + i
        ,'caller_name': callerName
    }));
  }
}

Paraxip.config.boards.editFxsAttHandler = function() {
  var fxsAttRecord;
  Paraxip.config.boards.fxsPhoneNumbers = [];
  Paraxip.config.boards.fxsCallerNames = [];

  var numIfs = parseInt(fxsInterfacesNumber.getRawValue(), 10) +
    parseInt(fxoInterfacesNumber.getRawValue(), 10);

  for( var i = 0; i < numIfs; i++ ) {
    fxsAttRecord = Paraxip.config.boards.fxsAttStore.getAt(i);

    for( j=0; j <= numIfs; j++) {
      if( Paraxip.config.boards.fxsPhoneNumbers[j] ==
          fxsAttRecord.data.phone_number ) {
        Ext.Msg.alert(
          "Duplicated Phone Numbers",
          "Two interfaces have the same phone number (" +
          Paraxip.config.boards.fxsPhoneNumbers[j] +
          "). Each analog interfaces must have a unique phone number."
        );
        return;
      }
    }
    Paraxip.config.boards.fxsPhoneNumbers[i] = fxsAttRecord.data.phone_number;
    Paraxip.config.boards.fxsCallerNames[i] = fxsAttRecord.data.caller_name;
  }

  Paraxip.config.boards.editFxsWindow.hide();
  addBoard();
}

Paraxip.config.boards.addBriModTypeFields = function() {
  var i = 1;
  Paraxip.config.boards.briAttStore.removeAll();
  for( i ; i <= briInterfacesNumber.getRawValue(); ++i ) {
    Paraxip.config.boards.briAttStore.add( new FxsAttDataRecord({
      'interface_id':i
      ,'briModType': "TERMINAL"
    }));
  }
}
Paraxip.config.boards.editBriAttHandler = function() {
  Paraxip.config.boards.briModTypeList = [];
  var i = 0;
  var j = 0;

  for( i; i < briInterfacesNumber.getRawValue(); i++ ) {
    var briAttRecord = Paraxip.config.boards.briAttStore.getAt(i);
    Paraxip.config.boards.briModTypeList[i] = briAttRecord.data.briModType;
  }
  
  addBoard();

  Paraxip.config.boards.editBriWindow.hide();

}

// Warn the user that he just added FXS interfaces and that in order for those
// interfaces to be reached, he needs to add their phone number in the FXS
// routing rule (by default called default_analog_fxs_pstn_out)
var addedAnalogFxsInterfacesMsg = function() {
  var callBack = warningMessagesList.pop();
  Ext.Msg.alert(
    "Routing rules not updated",
    "The analog FXS interfaces phone numbers " +
    "need to be added to the routing rule for analog channels. If you " +
    "have not changed it, the default name of that rule is " +
    "<b>default_analog_fxs_pstn_out</b>." +
    "<br><br>" +
    "If you already added analog FXS interfaces with the Quick Setup, the " +
    "value of the parameter sip.in.requestUri.canonical in that rule "+
    "will be set to <b>xxx|yyy|zzz|...</b> where xxx, yyy and zzz are the " +
    "phone numbers of the analog FXS interfaces you entered in the Quick " +
    "Setup. You just need to add the phone numbers of the new analog " +
    "FXS interfaces there. " +
    "<br><br>" +
    "If you have not added analog FXS interfaces with the Quick Setup, " +
    "locate the <b>XXX_FXS_PHONE_NUMBERS_XXX</b> token in this rule and " +
    "replace it with <b>xxx|yyy|zzz|...</b> where again, xxx yyy and zzz " +
    "are the new phone numbers for analog FXS interfaces. " +
    "<br><br>" +
    "If you edit the phone numbers manually for any of the analog " +
    "FXS interfaces, you need to go change the value in this routing " +
    "rule as well.",
    callBack
  );
}
// Warn the user that he just added FXO interfaces and that he need to tune 
// these interfaces (with FXO Tune tool)
var addedFxoInterfacesMsg = function() {
  var callBack = warningMessagesList.pop();
  Ext.Msg.alert(
    "You MUST run the calibration on each FXO interfaces",
    "In order to run the FXO Calibration, open the FXO Calibration Panel " +
    "(just under the FXO Calibration tab).",
    callBack
  );
}

var warningMessagesList = Array();
var successfullyModifiedBoards = function(hasFxsInterfaces, hasFxoInterfaces, addedBoard){

  Paraxip.isConfigOutOfSynch(true);
  reloadTree(true);
 
  if (addedBoard) {
    var msg = "The board(s) you just added is(are) not assigned to any config. " +
              "Please manually assign board(s) to a physical config and a " +
              "call control config."

    warningMessagesList = Array();

    if( hasFxsInterfaces ) {
      warningMessagesList.push(addedAnalogFxsInterfacesMsg);
    }
    if( hasFxoInterfaces ) {
      warningMessagesList.push(addedFxoInterfacesMsg);
    }
    var callBack = warningMessagesList.pop();
    Ext.Msg.alert("Board(s) not assigned to a config!",msg, callBack);
  }
  
  if(Paraxip.config.fxoTune.update){
    Paraxip.config.fxoTune.update();
  }
}

// When adding boards manually, this is the methods that will send the data
// of the board to add to the backend server.
var addBoard = function(){
  //submit information
  var form = addBoardForm.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "boards/addBoardFromEditor";
  el.dom.method = 'POST';
  var newBoardData = Paraxip.getItemsValue(addBoardForm.items);
  var isAnalog = Paraxip.isBoardAnalog(newBoardData['boardModel']);
  var isBRI = Paraxip.isBoardBRI(newBoardData['boardModel']);

  Paraxip.console.debug("New board data: " + newBoardData);

  if( isAnalog ) {
    var interfaceNum = parseInt(fxoInterfacesNumber.getRawValue(),10) +
                        parseInt(fxsInterfacesNumber.getRawValue(),10);
    newBoardData["interfaceNum"] = interfaceNum;
    newBoardData["phoneNumbersArray"] = Paraxip.config.boards.fxsPhoneNumbers;
    newBoardData["callerNamesArray"] = Paraxip.config.boards.fxsCallerNames;
  }
  else if(isBRI){
    newBoardData["briModType"] = Paraxip.config.boards.briModTypeList;
  }
  

  form.submit({
    params:{
      'newBoardData'  : Ext.util.JSON.encode(newBoardData)
    }
    ,success: function() {
      successfullyModifiedBoards(
        isAnalog && parseInt(fxsInterfacesNumber.getRawValue(),10), 
        isAnalog && parseInt(fxoInterfacesNumber.getRawValue(),10),
        true);
    }
    ,failure: Paraxip.failureMessage
  });

  addBoardWindow.hide();
};

// When adding or removing board(s) using the hardware detection mode, this is
// the method that will send the data of the boards to the backend server.
var addRemoveBoards = function() {
  // save current interfaces store values into the interfaces map
  var interfacesRecords = interfacesStore.getRange();

  if( interfacesRecords.length > 0 ) {
    Paraxip.config.boards.interfacesStoreMap.put(
      interfacesStore.lastOptions.boardID
      ,interfacesRecords
    );
  }

  var boardsData = [];
  var interfacesData = [];

  // add boards data
  var records = availableBoardsGrid.getStore().getRange();
  for (i = 0; i < records.length; i++) {
    // If one record has an invalid field, return false
    if( !isRecordValid(records[i]) ) {
      return false;
    }
    
    if (typeof records[i].data['phoneNumber'] == "string")
      var phoneNumerAsInt = parseInt( records[i].data['phoneNumber'] )
      records[i].data['phoneNumber'] = phoneNumerAsInt
    
    // If the interfaces for this board were not loaded, do not add it to the
    // list of boards
    if( Paraxip.config.boards.interfacesStoreMap.get(records[i].data.id) == null ) {
      continue;
    }

    boardsData.push(records[i].data);
  }
  boardsData = Ext.util.JSON.encode(boardsData);


  // add interfaces data
  var keySet = Paraxip.config.boards.interfacesStoreMap.keySet();
  var i = 0;
  for( i; i < Paraxip.config.boards.interfacesStoreMap.size(); ++i ) {
    if( keySet[i] == undefined ) { continue; }

    records = Paraxip.config.boards.interfacesStoreMap.get(keySet[i]);
    var recordsData = [];
    var recordsActive = false;
    for( var j = 0; j < records.length; ++j ) {
      if(boardActive(availableBoardsGrid, keySet[i]) &&
         !isRecordValid(records[j]) ) {
        return false;
      }
      recordsData.push(records[j].data);
      if (records[j].data.active) {
        recordsActive = true;
      }
    }
    if (!recordsActive) {
      Ext.Msg.alert("Interface selection",
                    "At least one interface must be selected on board with ID "
                     + keySet[i]);
      return false;
    }
    if( !Paraxip.validatePhoneNumbers(records) ) {
      return false;
    }

    interfacesData.push({
      "boardID":keySet[i]
      ,"interfacesData":recordsData
    });
  }
  interfacesData = Ext.util.JSON.encode(interfacesData);

  Ext.Ajax.request({
    url: Paraxip.baseUrl + "boards/addRemoveBoards"
    ,method: 'POST'
    ,disableCaching: false
    ,params: {
      "boards":boardsData
      ,"interfaces":interfacesData
    }
    ,success: function (response) {
      response = Ext.util.JSON.decode(response.responseText);
      successfullyModifiedBoards(response.data.fxsAdded, 
                                 response.data.fxoAdded, 
                                 response.data.addedBoard);
    }
    ,failure: Paraxip.failureMessage
  })

  // close the Add Board(s) window
  addBoardWindow.hide();
}

//-----------------------------------------------
// Boards Grid
//-----------------------------------------------
// create the Grid
var gridBoards = new Ext.grid.GridPanel({
  id      : "gridBoards"
  ,layout : 'fit'
  ,store  :  new Ext.data.JsonStore({
   id: 'storeBoards'
   ,url: Paraxip.baseUrl + "boards/getGridBoardsData"
   ,root: 'data'
   ,fields: [
     {name: 'id'}
     ,{name: 'displayName'}
     ,{name: 'type'}
     ,{name: 'model'}
     ,{name: 'numInterfaces'}
     ,{name: 'nodeId'} //Not displayed
   ]
  })
  ,columns : [
    {
      header: "ID"
      ,width: 35
      ,sortable: true
      ,dataIndex: 'id'
      ,css:'padding-top:5px;padding-bottom:5px;'
    },
    {
      id:'displayName'
      ,header: "Name"
      ,width: 100
      ,sortable: true
      ,dataIndex: 'displayName'
      ,css:'padding-top:5px;padding-bottom:5px;'
    },
    {
      header: "Type"
      ,width: 100
      ,sortable: true
      ,dataIndex: 'type'
      ,css:'padding-top:5px;padding-bottom:5px;'
    },
    {
      header: "Model"
      ,width: 100
      ,sortable: true
      ,dataIndex: 'model'
      ,css:'padding-top:5px;padding-bottom:5px;'
    },
    {
      header: "Number of Interfaces"
      ,width: 125
      ,sortable: true
      ,dataIndex:'numInterfaces'
      ,css:'padding-top:5px;padding-bottom:5px;'
    }
  ]
  ,stripeRows: true
  ,autoExpandColumn: 'displayName'
  ,title:'Boards Grid'
  ,viewConfig:{forcefit:true}
});

gridBoards.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

//Add the buttons to the footer of the grid
// Add and delete button are not ready. The difficulty is in the delete since
// it requires removal of assigned interfaces in the various pannel (physical
// config, isdn config, etc.)
if( hardwareDetectionMode ) {
  gridBoards.addButton("Add/Remove Board(s)",addBoardHandler,gridBoards);
} else {
  gridBoards.addButton("Add Board",addBoardHandler,gridBoards);
  gridBoards.addButton("Remove Board(s)",deleteBoardHandler,gridBoards);
}

//-----------------------------------------------
// Add Boards
//-----------------------------------------------

//
// FXS attributes objects and methods
//
Paraxip.config.boards.fxsAttData = [];

Paraxip.config.boards.FxsAttDataRecord = Ext.data.Record.create([
  {name: 'interface_id',type: 'int'}
  ,{name: 'phone_number'}
  ,{name: 'caller_name'}
]);

//Associates the data values with the columns of the grid
Paraxip.config.boards.fxsAttDataReader =
  new Ext.data.ArrayReader({}, Paraxip.config.boards.FxsAttDataRecord);

//Store that associates the data array
//to our array reader
Paraxip.config.boards.fxsAttStore = new Ext.data.Store({
  data: Paraxip.config.boards.fxsAttData
  ,reader: Paraxip.config.boards.fxsAttDataReader
});

Paraxip.config.boards.fxsAttGrid = Paraxip.createFxsAttributesGrid({
  id: "Paraxip.config.boards.fxsAttGrid"
  ,store: Paraxip.config.boards.fxsAttStore
});

Paraxip.config.boards.fxsAttGrid.addButton({
  id:"Paraxip.config.boards.editFxsAttButton"
  ,text:"Save"
}, Paraxip.config.boards.editFxsAttHandler);

Paraxip.config.boards.fxsAttGrid.addButton({
  id:"Paraxip.config.boards.cancelFxsAttButton"
  ,text:"Cancel"
}, function(){Paraxip.config.boards.editFxsWindow.hide();});

//
//BRI attributes objects and methods
//
Paraxip.config.boards.briAttData = [];

Paraxip.config.boards.BriAttDataRecord = Ext.data.Record.create([
{name: 'interface_id',type: 'int'}
,{name: 'briModType'}
]);

//Associates the data values with the columns of the grid
Paraxip.config.boards.briAttDataReader =
new Ext.data.ArrayReader({}, Paraxip.config.boards.BriAttDataRecord);

//Store that associates the data array
//to our array reader
Paraxip.config.boards.briAttStore = new Ext.data.Store({
data: Paraxip.config.boards.briAttData
,reader: Paraxip.config.boards.briAttDataReader
});

Paraxip.config.boards.briAttGrid = Paraxip.createBriAttributesGrid({
id: "Paraxip.config.boards.briAttGrid"
,store: Paraxip.config.boards.briAttStore
});

Paraxip.config.boards.briAttGrid.addButton({
  id:"Paraxip.config.boards.editBriAttButton"
  ,text:"Save"
}, Paraxip.config.boards.editBriAttHandler);

Paraxip.config.boards.briAttGrid.addButton({
  id:"Paraxip.config.boards.cancelBriAttButton"
  ,text:"Cancel"
}, function(){Paraxip.config.boards.editBriWindow.hide();});


Paraxip.config.boards.editFxsWindow = new Ext.Window({
  title     : 'Edit FXS interfaces attributes'
  ,layout   : 'border'
  ,width    : 505
  ,height   : 400
  ,plain    : true
  ,closable : false
  ,id       : "Paraxip.config.boards.editFxsWindow"
});
Paraxip.config.boards.editFxsWindow.add( Paraxip.config.boards.fxsAttGrid );

Paraxip.config.boards.editBriWindow = new Ext.Window({
  title     : 'Edit BRI interfaces attributes'
  ,layout   : 'border'
  ,width    : 505
  ,height   : 400
  ,plain    : true
  ,closable : false
  ,id       : "Paraxip.config.boards.editBriWindow"
});
Paraxip.config.boards.editBriWindow.add( Paraxip.config.boards.briAttGrid );



//
// Board form Panels attributes
//
var fxsInterfacesNumber = Paraxip.createAnalogInterfaceNumComboBox({
  id:"Paraxip.config.boards.analogFxsInterfaceNumField"
  ,fieldLabel: "Number of FXS Interfaces"
  ,name: "numFxsInterfaces"
});

var fxoInterfacesNumber = Paraxip.createAnalogInterfaceNumComboBox({
  id:"Paraxip.config.boards.analogFxoInterfaceNumField"
  ,fieldLabel: "Number of FXO Interfaces"
  ,name: "numFxoInterfaces"
});
var briInterfacesNumber = Paraxip.createBRIInterfaceNumComboBox({
  id: 'Paraxip.wizard.briInterfacesNumber'
  ,fieldLabel: "Number of BRI Interfaces"
  ,name: "briInterfacesNumber"
});

Paraxip.config.boards.fxsPhoneNumbers = [];

Paraxip.config.boards.fxsCallerNames = [];
Paraxip.config.boards.briModTypeList = [];

var interfaces_num_virtual = Paraxip.createVirtualInterfaceNumField({
  id:"Paraxip.config.boards.virtualInterfaceNum"
});

//
//Selection items
//
var boardSelection = Paraxip.createBoardSelectionComboBox({
  id:"Paraxip.config.boards.boardSelectionCB"
});

if( hardwareDetectionMode ) {

} else {
  boardSelection.on(
    'select',
    Paraxip.showInterfaceNumberListener.createDelegate(
      boardSelection,
      [fxoInterfacesNumber, 
       fxsInterfacesNumber, 
       interfaces_num_virtual, 
       briInterfacesNumber]
    )
  );
}

var config = {
  title         : 'Add Board'
  ,closeAction  : 'hide'
  ,plain        : true
  ,id           : "Paraxip.config.boards.addBoardWindow"
}

if( hardwareDetectionMode ) {
  config.layout = 'column';
  config.width = 950;
  config.height = 450;

} else {
  var addBoardForm = Paraxip.createAddBoardPanel({
    id: "Paraxip.config.boards.addBoardForm"
  });

  addBoardForm.add(boardSelection);
  addBoardForm.add(fxoInterfacesNumber);
  addBoardForm.add(fxsInterfacesNumber);
  addBoardForm.add(interfaces_num_virtual);
  addBoardForm.add(briInterfacesNumber);

  addBoardForm.on(
    'afterlayout',
    Paraxip.showInterfaceNumberListener.createDelegate(
      boardSelection,
      [fxoInterfacesNumber, 
       fxsInterfacesNumber, 
       interfaces_num_virtual, 
       briInterfacesNumber]
    )
  );

  config.layout = 'border';
  config.width = 505;
  config.height = 200;
}

// Add Board Window declaration
var addBoardWindow = new Ext.Window(config);

// If we are using the hardware detection mode, the attributes that we need to
// add to the form are not the same. In that case, the user will see two grids,
// one representing the boards available on the system, the other, the
// interfaces available for a board when it is selected.
if( hardwareDetectionMode ){
  addBoardWindow.addButton("Save", addRemoveBoards, gridBoards);
  addBoardWindow.addButton("Cancel", formCancel, addBoardWindow);

  // Array that contains the stores of intefaces for every bound found on the
  // system using netborder-list-boards tool. Used only when hardware detection
  // mode is enabled.
  Paraxip.config.boards.interfacesStoreMap = new Paraxip.Map();

  var interfacesStore = Paraxip.createAvailableInterfacesStore({
    id: "configInterfacesStore"
  }, Paraxip.config.boards.interfacesStoreMap);

  var beforeDeselectMethod = function(sm, index, row) {
    var boardID = sm.grid.store.lastOptions.params.boardID;

    var boardRecordIdx = boardsStore.find('id',boardID);
    var boardRecord = boardsStore.getAt(boardRecordIdx);

    if( boardRecord !== undefined && row.data.id == boardRecord.data.primaryClockReference ) {
      Ext.Msg.alert(
        "Trying to remove Primary Clock Reference Interface",
        "The interface with ID [" + row.data.id + "] is set as the Primary " +
        "Clock Reference for this board. Please change the Primary Clock " +
        "Reference of this board in the Board Details section before you " +
        "remove this interface."
      );

      return false;
    }

    return true;
  }

  var availableInterfacesGrid = Paraxip.createAvailableInterfacesGrid({
    id: "Paraxip.config.boards.availableInterfacesGrid"
    ,store: interfacesStore
    ,beforeDeselectMethod: beforeDeselectMethod
  });

  var boardsStore = Paraxip.createAvailableBoardsStore({
    id: "configBoardsStore"
  });

  boardsStore.on("loadException",function(el, o, response, e){
    Paraxip.console.debug("Unable to load available boards.");
  });

  var availableBoardsGrid = Paraxip.createAvailableBoardsGrid({
    id: "Paraxip.config.boards.availableBoardsGrid"
    ,store: boardsStore
  }, availableInterfacesGrid);

  addBoardWindow.add(availableBoardsGrid);

  addBoardWindow.add(availableInterfacesGrid);

} else {
  addBoardWindow.add( addBoardForm );
  addBoardForm.addButton("Save", Paraxip.config.boards.hasFxsInterfaces, gridBoards);
  addBoardForm.addButton("Cancel", formCancel, addBoardWindow);
}

//-----------------------------------------------
// Digital Boards Details
//-----------------------------------------------
var primaryClockReference = new Ext.form.ComboBox({
  id: "Paraxip.config.boards.primaryClockReference"
  ,fieldLabel: "Primary Clock Reference"
  ,name: "primaryClockReference"
  ,helpText: "sangomaDigitalBoardsangomaDigitalBoardprimaryClockReference"
  ,forceSelection: true
  ,triggerAction: 'all'
  ,mode: 'local'
  ,selectOnFocus: true
  ,valueField: 'id'
  ,displayField: 'id'
  ,store: new Ext.data.JsonStore({
    id: "primaryClockReferenceStore"
    ,url: Paraxip.baseUrl + "call/getInterfaces"
    ,root: 'data'
    ,fields: ['id','displayName']
    ,listeners: {
      "loadexception":function(dataProxy, type, action, options, response, arg){
        Paraxip.console.debug(
          "Primary Clock Reference load exception: " + dataProxy);
      }
    }
  })
});


var interfaceNumberingType = new Ext.form.ComboBox({
  id: "Paraxip.config.boards.interfaceNumberingType"
  ,fieldLabel: "Interface Numbering Type"
  ,name: "interfaceNumberingType"
  ,helpText: "sangomaDigitalBoardsangomaDigitalBoardinterfaceNumberingType"
  ,forceSelection: true
  ,triggerAction: 'all'
  ,mode: 'local'
  ,selectOnFocus: true
  ,store: ['DEFAULT','LINEAR']
});



var frmDigitalBoard = new ParaxipFormPanelWithHelp({
  id            : 'frmDigitalBoard'
  ,title        : 'Boards Details'
  ,labelWidth   : 75 // label settings here cascade unless overridden
  ,layout       : 'form'
  ,frame        : true
  ,bodyStyle    : 'padding:5px 5px 0'
  ,width        : 350
  ,labelPad     : 20
  ,autoScroll   : true
  ,helpText     : "sangomaDigitalBoardsangomaDigitalBoard"
  ,layoutConfig : {
    labelSeparator: ''
  }
  ,defaults     : {
    width: 230
    ,msgTarget: 'side'
  }
  ,defaultType  : 'textfield'
  ,items        : [{
    name: 'nodeId'
    ,hidden: true
    ,hideLabel: true
  },{
    id: "Paraxip.config.boards.digitalBoardID"
    ,fieldLabel: 'ID'
    ,name: 'id'
    ,helpText : "sangomaBoardCommonAttributesID"
    ,disabled: true
  },{
    id: "Paraxip.config.boards.digitalDisplayName"
    ,fieldLabel: 'Name'
    ,name: 'displayName'
    ,helpText : "sangomaBoardCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.boards.digitalType"
    ,fieldLabel: 'Type'
    ,name: 'type'
    ,disabled: true
  },{
    id: "Paraxip.config.boards.digitalModel"
    ,fieldLabel: 'Model'
    ,name: 'model'
    ,disabled: true
  },{
    id: "Paraxip.config.boards.digitalNumInterfaces"
    ,fieldLabel: 'Number of Interfaces'
    ,name: 'numInterfaces'
    ,disabled: true
  }
    ,primaryClockReference
    ,interfaceNumberingType
  ]
  ,buttons      : [{
    id: "Paraxip.config.boards.digitalBoardSaveButton"
    ,text: 'Save'
    ,handler: saveBoard
  },{
    id: "Paraxip.config.boards.digitalBoardCancelButton"
    ,text: 'Cancel'
    ,handler: boardCancelFunc
  }]
});

var panelDigitalBoard = new Ext.Panel({
  id            : 'panelDigitalBoard'
  ,region       : 'center'
  ,layout       : 'accordion'
  ,border       : false
  ,closable     : false
  ,buttonAlign  : 'right'
});

panelDigitalBoard.add(frmDigitalBoard);
panelDigitalBoard.add(gridInterfacesDigital);

//-----------------------------------------------
// Analog Boards Details
//-----------------------------------------------
var frmAnalogBoard = new ParaxipFormPanelWithHelp({
  id            : 'frmAnalogBoard'
  ,title        : 'Boards Details'
  ,labelWidth   : 75 // label settings here cascade unless overridden
  ,layout       : 'form'
  ,frame        : true
  ,bodyStyle    : 'padding:5px 5px 0'
  ,width        : 350
  ,labelPad     : 20
  ,autoScroll   : true
  ,helpText     : "sangomaAnalogBoardsangomaAnalogBoard"
  ,layoutConfig : {
    labelSeparator: ''
  }
  ,defaults     : {
    width: 230,
    msgTarget: 'side'
  }
  ,defaultType  : 'textfield'
  ,items        : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.boards.analogBoardID",
    fieldLabel: 'ID',
    name: 'id',
    disabled: true,
    helpText: "sangomaBoardCommonAttributesID"
  },{
    id: "Paraxip.config.boards.analogDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText: "sangomaBoardCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.boards.analogType",
    fieldLabel: 'Type',
    name: 'type',
    disabled: true
  },{
    id: "Paraxip.config.boards.analogModel",
    fieldLabel: 'Model',
    name: 'model',
    disabled: true
  },{
    id: "Paraxip.config.boards.analogNumInterfaces",
    fieldLabel: 'Number of Interface',
    name: 'numInterfaces',
    disabled: true
  },{
    id: "Paraxip.config.boards.analogWanpipeNumber"
    ,xtype: 'numberfield'
    ,fieldLabel: 'Wanpipe'
    ,name: 'wanpipe'
    ,helpText: "sangomaAnalogBoardsangomaAnalogBoardwanpipe"
  }]
  ,buttons     : [{
    id: "Paraxip.config.boards.analogBoardSaveButton",
    text: 'Save',
    handler: saveBoard
  },{
    id: "Paraxip.config.boards.analogBoardCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var panelAnalogBoard = new Ext.Panel({
  id           : 'panelAnalogBoard',
  region       : 'center',
  layout       : 'accordion',
  border       : false,
  closable     : false,
  buttonAlign  : 'right'
});

panelAnalogBoard.add(frmAnalogBoard);
panelAnalogBoard.add(gridInterfacesAnalog);
