//------------------------------------------------
// Factory to create common fields between 
// call control forms
//------------------------------------------------
Ext.namespace("Paraxip.config.callControl");

function createIdText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "ID",
    name        : "id",
    width       : 162,
    disabled    : true
  }, config));
};

function createNameText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "Display Name",
    name        : "displayName",
    width       : 162
  }, config));
};

function createMinimumOnHookDurationMs(config){
  return new Ext.form.NumberField(Ext.apply({
    fieldLabel    : "Minimum ON-HOOK Duration (ms)",
    name          : "minimumOnHookDurationMs",
    allowNegative : false,
    allowBlank    : false,
    allowDecimals : false
  }, config));
};

function createIsdnTerminationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'termination',
    fieldLabel      : "Termination",
    helpText        : "isdnCommonParameterstermination",
    store           : ['NETWORK','TERMINAL'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

// Note: Display name is NET5 but underlying data is set to ETSI.
var variantStoreFAS = new Ext.data.SimpleStore({  
  id      : "Paraxip.config.callControl.variantStore",
  fields  : ['value', 'variant_name'],
  data    : [
    ['4ESS', '4ESS'],
    ['5ESS', '5ESS'],    
    ['DMS100', 'DMS100'],
    ['ETSI', 'NET5 (ETSI, EuroISDN)'],          
    ['NI2', 'NI2'],
    ['INS1500', 'INS1500 (NTT-Japan)']
  ]
});
var variantStoreNFAS = new Ext.data.SimpleStore({  
  id      : "Paraxip.config.callControl.variantStore",
  fields  : ['value', 'variant_name'],
  data    : [
    ['4ESS', '4ESS']
    ,['5ESS', '5ESS']
    ,['DMS100', 'DMS100']
    ,['NI2', 'NI2']
  ]
});
var variantStoreBRI = new Ext.data.SimpleStore({  
  id      : "Paraxip.config.callControl.variantStore",
  fields  : ['value', 'variant_name'],
  data    : [
    ['ETSI', 'NET3 (ETSI, EuroISDN)']
  ]
});

function createIsdnSwitchVariantSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'switchVariant',
    fieldLabel      : "Switch Variant",
    helpText        : "isdnCommonParametersswitchVariant",
    displayField    : 'variant_name',
    valueField      : 'value',
    hiddenName      : 'switchVariant',
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,    
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnBChannelNegotiationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'BChannelNegotiation',
    fieldLabel      : "B-Channel Negotiation",
    helpText        : "isdnCommonParametersBChannelNegotiation",
    store           : ['EXCLUSIVE'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInitRestartProcSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'initiateRestartProcedure',
    fieldLabel      : "Initiate Restart Procedure",
    helpText        : "isdnCommonParametersinitiateRestartProcedure",
    store           : ['YES','NO'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInProgTonesGenSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'inbandProgressTonesGeneration',
    fieldLabel      : "Inband Progress Tones Generation",
    helpText        : "isdnCommonParametersinbandProgressTonesGeneration",
    store           : ['NEVER','ALWAYS','AS-NEEDED'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createIsdnInProgTonesIndSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : 'inbandProgressTonesIndicator',
    fieldLabel      : "Inband Progress Tones Indicator",
    helpText        : "isdnCommonParametersinbandProgressTonesIndicator",
    store           : ['ALERTING','PROGRESS','BOTH'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createCallerNameLocationSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "callerNameLocation",
    fieldLabel      : "Caller Name Location Method",
    helpText        : "isdnCommonParameterscallerNameLocation",
    store           : ['IN-SETUP-MSG-USING-DISPLAY-IE','IN-SETUP-MSG-USING-FACILITY-IE','IN-FACILITY-MSG-USING-FACILITY-IE','DISABLED'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createaWaitFacilityDelayMsNumberField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "waitFacilityDelayMs",
    fieldLabel      : "Wait Facility Delay (ms)",
    helpText        : "isdnCommonParameterswaitFacilityDelayMs",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createBriTopologySelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "topology",
    fieldLabel      : "Topology",
    helpText        : "briConfigPatternbriConfigtopology",
    store           : ['MULTIPOINT','POINT-TO-POINT'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createBriMultipleSubscriberNumbersText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "Multiple Subscriber Numbers",
    helpText    : "briConfigPatternbriConfigmultipleSubscriberNumbers",
    name        : "multipleSubscriberNumbers",
    width       : 162
  }, config));
};

function createBriTeiNumberField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "TEI",
    fieldLabel      : "Terminal Endpoint Identifier (TEI)",
    helpText        : "briConfigPatternbriConfigTEI",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    minValue        : 0,
    maxValue        : 63,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createAocBehaviorSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "aocBehavior",
    fieldLabel      : "Aoc Behavior",
    helpText        : "briConfigPatternbriConfigaocBehavior",
    store           : ['NONE','AOC-D','AOC-E'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};

function createServiceTypeSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "serviceType",
    fieldLabel      : "Service Type",
    helpText        : "briConfigPatternbriConfigserviceType",
    store           : ['VOICE','MODEM'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};



function createOverlapDigitsReceptionSelection(config){
  return new Ext.form.ComboBox( Ext.apply({
    name            : "overlapDigitsReception",
    fieldLabel      : "ISDN Overlap Digits Reception",
    helpText        : "isdnCommonParametersoverlapDigitsReception",
    store           : ['DISABLED','ENABLED'],
    forceSelection  : true,
    triggerAction   : 'all',
    mode            : 'local',
    typeAhead       : true,
    selectOnFocus   : true,
    width           : 145,
    minListWidth    : 145
  }, config));
};



function createOverlapDigitsReceptionMinDigitsField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "minNumberOfDigits",
    fieldLabel      : "Minimum Number Of Digits",
    helpText        : "isdnCommonParametersminNumberOfDigits",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    minValue        : 0,
    maxValue        : 32,
    width           : 145,
    minListWidth    : 145
  }, config));
};


function createOverlapDigitsReceptionMaxDigitsField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "maxNumberOfDigits",
    fieldLabel      : "Maximum Number Of Digits",
    helpText        : "isdnCommonParametersmaxNumberOfDigits",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    minValue        : 0,
    maxValue        : 32,
    width           : 145,
    minListWidth    : 145
  }, config));
};



function createOverlapDigitsReceptionEndCollectionDigitsText(config){
  return new Ext.form.TextField(Ext.apply({
    fieldLabel  : "End Collection Digits",
    helpText    : "isdnCommonParametersendCollectionDigits",
    name        : "endCollectionDigits",
    width       : 162
  }, config));
};


function createIsdnT310TimerField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "isdnT310Timer",
    fieldLabel      : "ISDN T310 Timer in Ms",
    helpText        : "isdnCommonParametersisdnT310Timer",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    minValue        : 10000,
    maxValue        : 300000,
    width           : 145,
    minListWidth    : 145
  }, config));
};


function createIsdnT203TimerField(config){
  return new Ext.form.NumberField( Ext.apply({
    name            : "isdnT203Timer",
    fieldLabel      : "ISDN T203 Timer in Ms",
    helpText        : "isdnCommonParametersisdnT203Timer",
    allowNegative   : false,
    allowBlank      : false,
    allowDecimal    : false,
    minValue        : 30000,
    maxValue        : 300000,
    width           : 145,
    minListWidth    : 145
  }, config));
};


//------------------------------------------------
//  Common methods
//------------------------------------------------

function callControlCancel(){
  reloadTree(true);
};


