//-----------------------------------------------
// FXO Calibration run calibration
//-----------------------------------------------
Ext.namespace("Paraxip.config.fxotune");

var calibrationStatusStore = new Ext.data.JsonStore( Ext.apply({
  root:"data.interfaces",
  fields: ['boardName','wanpipe','wanpipeInterface', 'calibrationStatus']
}));

calibrationStatusGrid = new Ext.grid.EditorGridPanel(
  Ext.apply({
    id: "Paraxip.config.fxotune.calibrationStatusGrid"
    ,store: calibrationStatusStore
    ,title: 'Calibration Status'
    ,region: "center"
    ,header: true
    ,height: 350
    ,width: '100%'
    ,enableColumnMove: false
    ,enableColumnHide: false
    ,enableColumnResize: false
    ,enableHdMenu: false
    ,columnWidth: .5
    ,columns: [
      {
        id: "boardName"
        ,header: "Board Name"
        ,width: 660
        ,sortable: false
        ,dataIndex: 'boardName'
        ,css: 'padding-top:5px;padding-bottom:5px;'
      },{
        id: 'wanpipe'
        ,header: "Wanpipe"
        ,width: 100
        ,sortable: false
        ,dataIndex: 'wanpipe'
        ,css: 'padding-top:5px;padding-bottom:5px;'
      },{
        id: "wanpipeInterface"
        ,header: "Wanpipe Interface"
        ,width: 100
        ,sortable: false
        ,hidden: false
        ,dataIndex: 'wanpipeInterface'
        ,css: 'padding-top:5px;padding-bottom:5px;'
      },{
        id: "calibrationStatus"
        ,header: "Calibration Status"
        ,width: 100
        ,sortable: false
        ,hidden: false
        ,dataIndex: 'calibrationStatus'
        ,css: 'padding-top:5px;'
      }
    ]
    ,stripeRows: true
    ,autoExpandColumn: 'boardName'
    ,viewConfig: {forcefit:true}
    ,style: 'text-align:left;'
  })
);

var fxotune_run_calibration = new Ext.Panel({
  id        : "Paraxip.config.fxotune_run_calibration"
  ,title    : "FXO Calibration - Run Calibration"
  ,frame    : true
  ,layout   : 'border'
  ,hideMode : 'offsets'
  ,header   : false
  ,items    : [{
    xtype: 'panel'
    ,region: 'west'
    ,layout: 'fit'
    ,margins: '5 0 0 0'
    ,cmargins: '5 5 0 0'
    ,width: 25
    ,height: 400
    ,minSize: 120
  },{
    xtype: 'panel'
    ,region: 'east'
    ,layout: 'fit'
    ,margins: '5 0 0 0'
    ,cmargins: '5 5 0 0'
    ,width: 25
    ,height: 400
    ,minSize: 120
  },{
    xtype: 'panel'
    ,region: 'north'
    ,layout: 'fit'
    ,margins: '5 0 0 0'
    ,cmargins: '5 5 0 0'
    ,width: 940
    ,height: 60
    ,minSize: 120
  },{
    xtype: 'panel'
    ,region: 'south'
    ,layout: 'fit'
    ,margins: '5 0 0 0'
    ,cmargins: '5 5 0 0'
    ,width: 940
    ,height: 8
    ,minSize: 120
  }]
  //,autoLoad  : Paraxip.baseUrl + "static/fxotune_first_panel.html"
  ,listeners: {beforeshow: function(){
  
    calibrationStatusGrid.store.removeAll();
    
    if(Paraxip.gatewayStarted){
      //prevent the user that the gateway will be stopped
      Ext.Msg.alert(
        'Gateway Stopped.',
        'In order to execute the calibration, the Gateway will be restarted.'
      );
    }

    runCalibration();
  }}
});


fxotune_run_calibration.add(calibrationStatusGrid);
var runCalibrationProgressBar;
var runCalibration = function() {
  
  fxoCalibration_cancelCalibration_button.enable();
  sm = fxoInterfacesGrid.getSelectionModel();
  interfacesSelected = sm.getSelections();
  
  var interfacesData = new Array();
  var i = 0;
  for( i; i < interfacesSelected.length; ++i ) {
    interfacesData.push(
        {'boardName':interfacesSelected[i].data.boardName,
         'wanpipe':interfacesSelected[i].data.wanpipe,
         'wanpipeInterface':interfacesSelected[i].data.wanpipeInterface });
  }
  
  interfacesDataJSON = Ext.util.JSON.encode(interfacesData);

  Ext.Ajax.request({
    url: Paraxip.baseUrl + "fxoTune/calibrate"
    ,method: 'POST'
    ,disableCaching: true
    ,params:{'interfacesList' : interfacesDataJSON}

    ,success: function(calibrateResult){
    
      calibrateResult = Ext.decode(calibrateResult.responseText)
      
      if(calibrateResult.data == "error_calibration_already_running")
      {
        Ext.Msg.alert(
          "FXO Calibration is already running",
          "FXO Calibration is already running. To start a new calibration, " +
          "please wait until this calibration is done (or you can also " +
          "cancel it)"
        );
      }
      
      if(calibrateResult.data == "error_fxotune_app_not_found")
      {
        Ext.Msg.alert("ERROR: FXO Tune Application Not Found", 
        "FXO Tune Application not found. The Calibration stopped. " +
        "Please look at your gateway properties and set a valid " +
        "netborder.sangoma.fxotune.appPath.",
        calibrationCompleted)
      }
      
        
      calibrationStatusTask = {
        run: function(){
          
          Ext.Ajax.request({
            url: Paraxip.baseUrl + "fxoTune/getCalibrationStatus"
            ,disableCaching: true
            ,success: function(result){
              var status = Ext.decode(result.responseText)
              
              if(status.data.general == 'completed') {
                calibrationCompleted();
              } else if( status.data.general == 'gateway_failed_to_start' ) {
                Ext.Msg.alert(
                  "FXO Calibration Failed to start the gateway",
                  "The Fxo-Calibration didn't start because the gateway " +
                  "failed to failed to launched. Look at the gataway logs.",
                  calibrationCompleted
                );
              } else if( status.data.general == 'canceled' ) {
                Ext.Msg.alert(
                  "FXO Calibration Canceled",
                  "The Calibration has been canceled",
                  calibrationCanceled
                );
              } else if( status.data.general == "failed" ) {
                Ext.Msg.alert(
                  "FXO Calibration Application failed",
                  "FXO calibration application failed. Please see Gateway " +
                  "Manager log file for more details.",
                  calibrationFailed
                );
              }
              //calibrationStatusStore.reload();

              calibrationStatusStore.loadData(status);
            }
          });
        },
        interval: 5000
      }
      Ext.TaskMgr.start(calibrationStatusTask);
      fxoCalibration_progressBar.wait();
    }
    
    ,failure: function(){
      alert("Calibration failed.");
    }

  });

};
var cancelProgressBar;
var cancelCalibration = function(){
  fxoCalibration_cancelCalibration_button.disable();
  
  Ext.Msg.show({
    title:'Cancel Calibration',
    msg: 'You are about to cancel the FXO Tuning. ' + 
         'The Cancellation can take up to 1 minute. Are you sure you want '+
         'to continue? ',
    buttons: Ext.Msg.OKCANCEL,
    fn: function(choice){
      if(choice == "ok"){
        sendCancelSignal();
        Ext.Msg.minProgressWidth=600;
        cancelProgressBar = Ext.Msg.wait(
        "Cancellation can take up to 1 minute.",
        "Calibration Cancelling");
      }
      else
      {
        fxoCalibration_cancelCalibration_button.enable();
      }
    },
    animEl: 'elId',
    icon: Ext.MessageBox.QUESTION
  });
  
};

var cancelSignalFailedCount = 0
var sendCancelSignal = function(){
  fxoCalibration_progressBar.reset();
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "fxoTune/cancelCalibration"
    ,disableCaching: true
    ,failure: function(result){
      cancelSignalFailedCount++;
      if(cancelSignalFailedCount < 10)
        sendCancelSignal();
      else
        cancelSignalFailedCount = 0;
    }
    ,success: function(result){
      cancelSignalFailedCount=0;
    }
    
  });
};

var finishCalibration = function(){
  calibrationFinished();
  
  var mainTab = Ext.getCmp('mainTabPanel');
  fxoTuningNavHandler("next");

  // We check that mainTabPanel is defined to see if the Quick Setup was
  // executed with the WebUI in its complete form or using the lightbox URL
  // http://127.0.0.1:7783/wizard/lightbox_quick_setup/. If the WebUI is in its
  // complete form, we go to the Status & Controls tab. Otherwise, we go to the
  // Finish form of the Quick Setup.
  if( mainTab != undefined ) {
    Paraxip.isConfigOutOfSynch(true);

    mainTab.setActiveTab("statusControlTabId");

    var editorTab = mainTab.findById("pstnEditorTabId");

    editorTab.setDisabled(false);

    //get the treePanel
    if (editorTab.rendered) {
      reloadTree(false);
    }

    Paraxip.mainTabPanel.configFileExists = true;
  } else {
    // since we are in a lightbox, redirect to Status & Controls using
    // window.location
    window.location = 
      Paraxip.baseUrl + "status/lightbox_controls?pingGW=false";
  }
  
};

var calibrationCanceled = function(){
  Ext.TaskMgr.stop(calibrationStatusTask);
  fxoCalibration_cancelCalibration_button.enable();
  //calibrationStatusStore.reload();
  cancelProgressBar.hide();
  fxoTuningNavHandler("back");
};
var calibrationFailed = function(){
  Ext.TaskMgr.stop(calibrationStatusTask);
  fxoCalibration_cancelCalibration_button.enable();
  //calibrationStatusStore.reload();
  fxoTuningNavHandler("back");
};

var calibrationCompleted = function(){
  Ext.TaskMgr.stop(calibrationStatusTask);
  fxoCalibration_progressBar.reset();
  fxoCalibration_progressBar.setVisible(false);
  fxoCalibration_cancelCalibration_button.setVisible(false);
  fxoCalibration_finishCalibration_button.setVisible(true);
};
var calibrationFinished = function(){
  fxoCalibration_finishCalibration_button.setVisible(false);
};

