/*
 * Paraxip Oam Common Library to Ext JS Library 2.2
 * Copyright(c) 2008, Paraxip Technologies
 */

Ext.namespace('Paraxip'
              ,'Paraxip.config'
              ,'Paraxip.config.routingRules'
              ,'Paraxip.status'
              ,'Paraxip.status.channels'
              ,'Paraxip.status.system'
              ,'Paraxip.status.controls');

//List of supported countries
Paraxip.countryList = [
    'USA',
    'CANADA',
    'FCC',
    'TBR21',
    'ARGENTINA',
    'AUSTRALIA',
    'AUSTRIA',
    'BAHRAIN',
    'BELGIUM',
    'BRAZIL',
    'BULGARIA',
    'CHILE',
    'CHINA',
    'COLUMBIA',
    'CROATIA',
    'CYPRUS',
    'CZECH',
    'DENMARK',
    'ECUADOR',
    'EGYPT',
    'ELSALVADOR',
    'FINLAND',
    'FRANCE',
    'GERMANY',
    'GREECE',
    'GUAM',
    'HONGKONG',
    'HUNGARY',
    'ICELAND',
    'INDIA',
    'INDONESIA',
    'IRELAND',
    'ISRAEL',
    'ITALY',
    'JAPAN',
    'JORDAN',
    'KAZAKHSTAN',
    'KUWAIT',
    'LATVIA',
    'LEBANON',
    'LUXEMBOURG',
    'MACAO',
    'MALAYSIA',
    'MALTA',
    'MEXICO',
    'MOROCCO',
    'NETHERLANDS',
    'NEWZEALAND',
    'NIGERIA',
    'NORWAY',
    'OMAN',
    'PAKISTAN',
    'PERU',
    'PHILIPPINES',
    'POLAND',
    'PORTUGAL',
    'ROMANIA',
    'RUSSIA',
    'SAUDIARABIA',
    'SINGAPORE',
    'SLOVAKIA',
    'SLOVENIA',
    'SOUTHAFRICA',
    'SOUTHKOREA',
    'SPAIN',
    'SWEDEN',
    'SWITZERLAND',
    'SYRIA',
    'TAIWAN',
    'THAILAND',
    'UAE',
    'UK',
    'YEMEN'
  ]

Paraxip.minAnalogInterfaces = 2;
Paraxip.maxAnalogInterfaces = 12;

Paraxip.minBriInterfaces = 2;
Paraxip.maxBriInterfaces = 12;


// Extract root of the url (e.g. http://localhost:7783/ in
// http://localhost:7783/wizard/lightbox_quick_setup)
var url = window.location.href;
// Find the location of the "//" characters
var start_from = url.indexOf("//");
// Start the search after the '//' characters
var end_at = url.indexOf("/",start_from+2);
// Include '/' in the url
Paraxip.baseUrl = url.substring(0,end_at+1);

/**
 * Generic function to search for an item into a list or
 * an array.
 * @param in_list: the list or array to look in
 * @param in_item: the item to look for
 * @return bool: true if the item was found; false if not
 */
Paraxip.isItemInList = function(in_list, in_item) {
  for( i in in_list ) {
    if( in_list[i] == in_item ) {
      return true;
    }
  }
  return false;
};

/*
 * Creates a HashMap as the one in Java for Javascript.
 * Use as followed:
 *  var myMap = Paraxip.Map();
 *  key = 'xxxx';
 *  value = 'yyyy';
 *  myMap.put(key,value);
 *  myMap.get(key);
 */
Paraxip.Map = function()
{
  // members
  this.keyArray = new Array(); // Keys
  this.valArray = new Array(); // Values

  // methods
  this.put = put;
  this.get = get;
  this.size = size;
  this.clear = clear;
  this.keySet = keySet;
  this.valSet = valSet;
  this.showMe = showMe;   // returns a string with all keys and values in map.
  this.findIt = findIt;
  this.remove = remove;
};

function put( key, val )
{
  var elementIndex = this.findIt( key );

  if( elementIndex == (-1) )
  {
    this.keyArray.push( key );
    this.valArray.push( val );
  }
  else
  {
    this.valArray[ elementIndex ] = val;
  }
};

function get( key )
{
  var result = null;
  var elementIndex = this.findIt( key );

  if( elementIndex != (-1) )
  {
    result = this.valArray[ elementIndex ];
  }

  return result;
};

function remove( key )
{
  var result = null;
  var elementIndex = this.findIt( key );

  if( elementIndex != (-1) )
  {
    this.keyArray = this.keyArray.removeAt(elementIndex);
    this.valArray = this.valArray.removeAt(elementIndex);
  }

  return ;
};

function size()
{
  return (this.keyArray.length);
};

function clear()
{
  for( var i = 0; i < this.keyArray.length; i++ )
  {
    this.keyArray.pop(); this.valArray.pop();
  }
};

function keySet()
{
  return (this.keyArray);
};

function valSet()
{
  return (this.valArray);
};

function showMe()
{
  var result = "";

  for( var i = 0; i < this.keyArray.length; i++ )
  {
    result += "Key: "+this.keyArray[i] + "\tValues: " + this.valArray[i] + "\n";
  }
  return result;
};

function findIt( key )
{
  var result = (-1);

  for( var i = 0; i < this.keyArray.length; i++ )
  {
    if( this.keyArray[ i ] == key )
    {
      result = i;
      break;
    }
  }
  return result;
};

function removeAt( index )
{
  var part1 = this.slice( 0, index);
  var part2 = this.slice( index+1 );

  return( part1.concat( part2 ) );
};

Array.prototype.removeAt = removeAt;

// end of Paraxip.Map methods

/*
 * Those methods scans the boardGrid and for each board, test if it is an analog
 * board, a digital board or a virtual_digital board. Depending on the method,
 * it returns true if one of the boards matches the disired type. To test the
 * board type, we use the isBoardAnalog, isBoardDigital and
 * isBoardVirtualDigital methods.
 */
Paraxip.gridHasAnalogBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardAnalog(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.gridHasDigitalBoard = function(grid) {

  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardDigital(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.gridHasPRIBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardPRI(board_type)) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.hwDetectionGridHasDigitalBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardDigital(board_type) && records[i].get('active') ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.hwDetectionGridHasPRIBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardPRI(board_type) && records[i].get('active') ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.hwDetectionGridHasBRIBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardBRI(board_type) && records[i].get('active') ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.gridHasVirtualDigitalBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardVirtualDigital(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

//If selected board is an A200 or an A400 or a B600, it is analog.
Paraxip.isBoardAnalog = function(boardType) {
  return ( boardType == "A200" || boardType == "A400" || boardType == "B600" || boardType == "B700_analog");
};

// Digital boards starts with "A1" or is an A500/B500/B700_digital
Paraxip.isBoardDigital = function(boardType) {
  return ((/^A1/).test(boardType) || boardType=="A500" || boardType=="B500" || boardType=="B700_digital") ;
};

Paraxip.isBoardPRI = function(boardType) {
  return ((/^A1/).test(boardType));
};

Paraxip.isBoardBRI = function(boardType) {
  return boardType=="A500" || boardType=="B500" || boardType=="B700_digital";
};

Paraxip.isBoardVirtualDigital = function(boardType) {
  return (boardType == "VIRTUAL_DIGITAL");
};

//gateway status list
Paraxip.activeStatus = {
  id      : "Paraxip.activeStatus"
  ,text    : "Gateway Status:  started"
  ,iconCls : ''
};

Paraxip.downStatus = {
  id      : "Paraxip.downStatus"
  ,text    : "Gateway Status:  stopped"
  ,iconCls : ''
};

Paraxip.stoppingStatus = {
  id    : "Paraxip.stoppingStatus"
  ,text  : "Gateway Status:  Shutting down"
};

Paraxip.serviceStartingStatus = {
  id    : "Paraxip.serviceStartingStatus"
  ,text  : "Gateway Status:  Starting Service"
}

Paraxip.serviceRestartingStatus = {
  id    : "Paraxip.serviceRestartingStatus"
  ,text  : "Gateway Status:  Restarting Service"
}

Paraxip.initializingApplicationStatus = {
  id    : "Paraxip.initializingApplicationStatus"
  ,text  : "Gateway Status: Service Started...Initializing Application"
  ,iconCls : ''
}

Paraxip.gatewayStarted; //keep trace of previous gateway status
Paraxip.serviceStarted;

Paraxip.isGwStartedCall = function() {
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "status/pingGateway",
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function (result)
    {
      var isGWStarted = Ext.util.JSON.decode(result.responseText).value;

      if( isGWStarted ) {
        if( isGWStarted != Paraxip.gatewayStarted ) {
          Paraxip.changeGwStatusBar(
            Paraxip.activeStatus,
            isGWStarted,
            !isGWStarted
          );
          Paraxip.disableItems(false);
        }
      } else {
        if( Paraxip.serviceStarted ) {
          Paraxip.changeGwStatusBar(
            Paraxip.initializingApplicationStatus,
            !isGWStarted,
            isGWStarted
          );
        } else {
          Paraxip.changeGwStatusBar(
            Paraxip.downStatus,
            isGWStarted,
            !isGWStarted
          );
        }
        Paraxip.disableItems(true);
      }
      Paraxip.gatewayStarted = isGWStarted;
    },
    failure: Paraxip.failureMessage
  });
};

/*
 * Change the Gateway Status: xxxx value in the status bar of the main panel
 * @param isStartDisabled: true if the start button should be grayed out; false
 *                         if not;
 * @param isStopDisabled: true if the stop button should be grayed out; false
 *                        if not;
 */
Paraxip.changeGwStatusBar = function(status, isStartDisabled, isStopDisabled) {
  if( !isGatewayInCLI ) {
    if( Paraxip.status.controls.startGwButton != undefined ) {
      Paraxip.status.controls.startGwButton.setDisabled(isStartDisabled);
      Paraxip.status.controls.stopGwButton.setDisabled(isStopDisabled);
      Paraxip.status.controls.restartGwButton.setDisabled(isStopDisabled);
    }
  }

  if( Paraxip.status.controls.messagePanel != undefined &&
      Paraxip.status.controls.messagePanel.body != undefined) {
    if( status == Paraxip.activeStatus ) {
      Paraxip.status.controls.messagePanel.body.load({
        url:Paraxip.baseUrl + "static/gateway_up.html",
        nocache:false
      });
    } else {
      Paraxip.status.controls.messagePanel.body.load({
        url:Paraxip.baseUrl + "static/gateway_down.html",
        nocache:false
      });
    }
  }

  var statusBar = Ext.getCmp('gw_status_bar');
  if( statusBar != undefined ) {
    if( status == Paraxip.stoppingStatus ||
        status == Paraxip.serviceStartingStatus ) {
      statusBar.showBusy(status);
    } else {
      statusBar.setStatus(status);
    }
  }
};

Paraxip.disableItems = function(isItemsDisabled){
  if( Paraxip.status.statusTabPanel != undefined ) {
    Paraxip.status.statusTabPanel.getItem('systemStatusTabId')
      .setDisabled(isItemsDisabled);
    Paraxip.status.statusTabPanel.getItem('channelsTabId')
      .setDisabled(isItemsDisabled);
    Paraxip.status.statusTabPanel.getItem('loggingTabId')
      .setDisabled(isItemsDisabled);
  }
  if( Paraxip.config.routingRules.validate != undefined ) {
    Paraxip.config.routingRules.validate.setDisabled(isItemsDisabled);
  }
  if( Paraxip.status.channels.channelGrid != undefined ) {
    Paraxip.status.channels.channelGrid.setDisabled(isItemsDisabled);
  }
};

/*
 * Change the status of the out of sync status bar to remind to the user he has
 * to restart the gateway in order to get the modifications made to the config
 * effective or if the modifications have been applied.
 * @param outOfSynch: true if the config is out of synch; false if not.
 */
Paraxip.isConfigOutOfSynch = function(outOfSynch) {
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "config/changeConfigOutOfSynch",
    method: 'POST',
    params: {newValue:outOfSynch}
  });
  if( Paraxip.mainTabPanel != undefined ) {
    if( outOfSynch ) {
      Paraxip.mainTabPanel.configSynchStatusBar
        .setStatus(Paraxip.mainTabPanel.outOfSynchText);
    } else {
      Paraxip.mainTabPanel.configSynchStatusBar
        .setStatus(Paraxip.mainTabPanel.synchText);
    }
  }
};

/*
 * Show a progress bar freezing all the ui until the request is finished.
 * @param msg: the message to display in the message window.
 */
Paraxip.showProgressBar = function(msg){
  Ext.MessageBox.show({
    msg: msg
    ,progressText: 'Starting...'
    ,width:300
    ,wait:true
    ,waitConfig: {interval:200}
    //icon:'ext-mb-download', //custom class in msg-box.html
    ,animEl: 'mb7'
 });
};

/**
 * Changes the text in the Progress Bar message box
 */
Paraxip.updateProgressBarText = function(msg){
  Ext.MessageBox.updateText(msg);
};

/*
 * Hide the progress bar window showed with the method Paraxip.showProgressBar
 */
Paraxip.hideProgressBar = function(){
  Ext.MessageBox.hide();
};

/*
 * Display a pop-up message telling the user his last operation failed.
 */
Paraxip.failureMessage = function(){
  Ext.Msg.alert(
    "Error!",
    "Your last operation failed. Please contact support if the problem persist."
  );
};

/*
 * The following methods create all the elements that will, once put
 * together, form the Add Board Panel used in the wizard's hardware
 * configuration section and in the editor's boards section.
 */
Paraxip.createBoardSelectionComboBox = function(config){
  var board_init_value =  'A101';

  //Selection items
  var boardModels = ['A101','A102','A104','A108','A116','A200','A500','B500','B600', 
                     'B700_digital', 'B700_analog'];
  if(typeof offerVirtualBoards != "undefined" && offerVirtualBoards == true) {
    boardModels.push('VIRTUAL_DIGITAL');
  }
  return new Ext.form.ComboBox( Ext.apply({
    fieldLabel      : "Board Type"
    ,name           : "boardModel"
    ,store          : boardModels
    ,forceSelection : true
    ,triggerAction  : 'all'
    ,mode           : 'local'
    ,typeAhead      : true
    ,value          : board_init_value
    ,selectOnFocus  : true
    ,width          : 130
    ,listeners      : {
      'render': function(){
        var elp = this.el.findParent('.x-form-element', 5, true);
        this.helpImage = elp.createChild({
          style : "width:16px;height:18px;position:relative;left:30px;" +
                  "top:-23px;display:block;background:transparent " +
                  "url(/static/images/help.gif) no-repeat 0 2px;" +
                  "cursor:pointer!important;"
        });

        this.helpImage.alignTo(this.wrap, 'tl-tr', [2, 0]);

        this.helpImage.on("click", function()
        {
          helpWindow.setTitle("Add Board Help");
          helpWindow.show();

          var textarea = helpWindow.getComponent("helpText_area");
          textarea.body.load({
            url:"static/help/quick-setup/qs-add-board.html"});
        }, this);//scope

      }
      ,'resize': function(){
        this.alignHelpImage();
      }
    }
  },config));
};

Paraxip.showInterfaceNumberListener
    = function(analogFxoIntfNum, analogFxsIntfNum, virtualIntfNum, briIntfNum){
  var selection = this.getValue();

  if( selection == "B600" ) {
    // Force the number of FXO interfaces to 4 and the value of FXS interfaces
    // to 1.
    analogFxoIntfNum.setValue(4);
    analogFxoIntfNum.enable();
    analogFxsIntfNum.setValue(1);
    analogFxsIntfNum.enable();

  } else {
    analogFxoIntfNum.setContainerVisible(Paraxip.isBoardAnalog(selection));
    analogFxsIntfNum.setContainerVisible(Paraxip.isBoardAnalog(selection));
    
  }
  
  briIntfNum.setContainerVisible(Paraxip.isBoardBRI(selection));
  virtualIntfNum.setContainerVisible(Paraxip.isBoardVirtualDigital(selection));
  return true;
};

Paraxip.createAnalogInterfaceNumComboBox = function(config){
return new Ext.form.ComboBox( Ext.apply({
    store           : [[0,'0'],[2,'2'],[4,'4'],[6,'6'],[8,'8'],[10,'10'],[12,'12']]
    ,forceSelection : true
    ,triggerAction  : 'all'
    ,mode           : 'local'
    ,typeAhead      : true
    ,value          : 0
    ,selectOnFocus  : true
    ,width          : 130
  },config));
};

Paraxip.createBRIInterfaceNumComboBox = function(config){
  return new Ext.form.ComboBox( Ext.apply({
      store           : [[0,'0'],[2,'2'],[4,'4'],[6,'6'],[8,'8'],[10,'10'],[12,'12']]
      ,forceSelection : true
      ,triggerAction  : 'all'
      ,mode           : 'local'
      ,typeAhead      : true
      ,value          : 0
      ,selectOnFocus  : true
      ,width          : 130
    },config));
  };
  
Paraxip.createFxsAttributesGrid = function(config){
  return new Ext.grid.EditorGridPanel( Ext.apply({
    layout    : 'fit'
    ,region   : 'center'
    ,style    : 'text-align:left;'
    ,frame    : true
    ,columns  : [
      {
        id: 'interfaceID'
        ,header: "Interface ID"
        ,width: 75
        ,dataIndex: 'interface_id'
      }
      ,{
        header: "Phone Number"
        ,width: 140
        ,dataIndex: 'phone_number'
        ,editor: new Ext.form.NumberField({
          allowBlank  : false
        })
      }
      ,{
        header: "Caller Name"
        ,width: 230
        ,dataIndex: 'caller_name'
        ,editor: new Ext.form.TextField({
          allowBlank  : false
        })
      }
    ]
    ,enableColumnResize  : true
    ,enableColumnMove    : false
    ,enableHdMenu        : false
    ,trackMouseOver      : true
    ,stripeRows          : false
    ,autoExpandColumn    : 'interfaceID'
    ,sm: new Ext.grid.RowSelectionModel({singleSelect:true})
  },config));
};
Paraxip.createBriAttributesGrid = function(config){
  return new Ext.grid.EditorGridPanel( Ext.apply({
    layout    : 'fit'
    ,region   : 'center'
    ,style    : 'text-align:left;'
    ,frame    : true
    ,columns  : [
      {
        id: 'interfaceID'
        ,header: "Interface ID"
        ,width: 75
        ,dataIndex: 'interface_id'
      }
      ,{
        header: "BRI Mod Type"
        ,width: 140
        ,dataIndex: 'briModType'
        ,editor:  new Ext.form.ComboBox({
          forceSelection  : true,
          triggerAction   : 'all',
          mode            : 'local',
          selectOnFocus   : true,
          valueField      : 'id',
          displayField    : 'id',
          store           : ['NETWORK', 'TERMINAL'],
          listClass       : 'x-combo-list-small'
        })
      }
      
    ]
    ,enableColumnResize  : true
    ,enableColumnMove    : false
    ,enableHdMenu        : false
    ,trackMouseOver      : true
    ,stripeRows          : false
    ,autoExpandColumn    : 'interfaceID'
    ,sm: new Ext.grid.RowSelectionModel({singleSelect:true})
  },config));
};


Paraxip.createVirtualInterfaceNumField = function(config){
  return new Ext.form.NumberField( Ext.apply({
    fieldLabel      : "Number of Virtual Interfaces (must be even)"
    ,disabled       : false
    ,hidden         : true
    ,name           : "interfaceNum"
    ,width          : 130
    ,value          : 2
    ,allowNegative  : false
    ,allowDecimals  : false
    ,allowBlank     : false
    ,regex          : /[02468]$/ // Must be even
    ,regexText      : 'Must use an even number'
  },config));
};

Paraxip.createBriInterfaceNumField = function(config){
  return new Ext.form.NumberField( Ext.apply({
    fieldLabel      : "Number of BRI Interfaces"
    ,disabled       : false
    ,hidden         : true
    ,name           : "briInterfaceNum"
    ,width          : 130
    ,value          : 2
    ,allowNegative  : false
    ,allowDecimals  : false
    ,allowBlank     : false
    ,regex          : /[02468]$/ // Must be even
    ,regexText      : 'Must use an even number'
  },config));
};

Paraxip.createAddBoardPanel = function(config){
  return new Ext.form.FormPanel( Ext.apply({
    labelWidth    : 175
    ,labelPad     : 10
    ,bodyStyle    : 'padding:10px;'
    ,region       : 'center'
    ,frame        : false
    ,header       : false
    ,footer       : false
    ,defaults     : {
      anchor: '75%'
    }
  },config));
};

Paraxip.createAvailableBoardsStore = function(config) {
  return new Ext.data.JsonStore( Ext.apply({
    url: Paraxip.baseUrl + "boards/getAvailableBoards"
    ,idProperty: 'id'
    ,root: 'data'   
    ,fields: [
      'id'
      ,'active'
      ,'displayName'
      ,'type'
      ,'model'
      ,'numInterfaces'
      ,'pciBus'
      ,'pciSlot'
      ,'wanpipe'
      ,'primaryClockReference'
    ]
    ,listeners: {
      "beforeload":function(store, options) {
        
        return true;
      }
      ,"load": function(store, records, options) {
        var patate = 1;

      }
      ,"update": function( store,  record,  operation ) {
        var brocoli = 2
      }
    }
  },config));
}

Paraxip.createAvailableInterfacesStore = function(config, storeMap) {
  return new Ext.data.JsonStore( Ext.apply({
    url: Paraxip.baseUrl + "boards/getAvailableInterfaces"
    ,root: 'data'
    ,idProperty: 'id'
    ,fields: [
      'id'
      ,'active'
      ,'displayName'
      ,'wanpipe'
      ,'wanpipeInterface'
      ,'type'
      ,{name:'phoneNumber', type:'int'}
      ,'callerName'
      ,'briModType'
    ]
    ,listeners: {
      "beforeload":function(store, options) {
        // If the records were already loaded, do load them again. It would
        // wipe the modifications made by the user that are not saved yet.
        if( store.getCount() != 0 ) {
          storeMap.put(
            store.lastOptions.params.boardID,
            store.getRange()
          );
        }

        if( storeMap.get(options.params.boardID) ) {
          store.removeAll();
          store.add(
            storeMap.get(options.params.boardID)
          );
          store.lastOptions.params.boardID = options.params.boardID;
          //Force the load event even if the ajax request won't be done
          store.fireEvent("load",store, store.getRange(), options);
          return false;
        }
        return true;
      }
      ,"load": function(store, records, options) {
        if( storeMap.get(options.params.boardID) == undefined ) {
          storeMap.put(options.params.boardID, records);
        }

      }
    }
  }, config));
}

var isRecordValid = function(record) {
  if( !record.data.active || 
      (record.data.type != "FXS" && record.data.type != "FXO") ) {return true;}

  if( record.data.displayName == "" ) {
    Ext.Msg.alert("Invalid field",
                  "Board/Interface with ID " + record.data.ID +
                  " must have a display name.");
    return false;
  }

  var strMessage = "";
  if( record.data.phoneNumber == "" ) {
    strMessage = "Analog Interface " + record.data.displayName + " must have ";
    if( record.data.type == "FXS" && record.data.callerName == "" ) {
      strMessage += "Caller Name and Phone Number.";
    } else {
      strMessage += "Phone Number.";
    }
  } else if( record.data.type == "FXS" && record.data.callerName == "" ) {
    strMessage = 
    "Analog Interface " + record.data.displayName + " must have Caller Name.";
  }

  if( strMessage != "" ) {
    Ext.Msg.alert("Invalid field", strMessage);
    return false;
  }

  if( record.data.type == "FXS" && 
      !isValidCallerName(record.data.callerName) ) {
    Ext.Msg.alert("Invalid field",
                  "Invalid character in caller name : " +
                  record.data.callerName + ". Acceptable characters are " +
                  "all the alph-numeric characters, -, _ and the white space");
    return false;
  }

  return true;
}

Paraxip.validatePhoneNumbers = function( records ) {
  for( var j = 0; j < records.length; ++j ) {
    if( records[j].data.active ) {
      recordsActive = true;
      if( records[j].data.type == "FXS" ) {
        for( var k = 0; k < records.length; ++k ) {
          if( records[k].data.active &&
              j != k &&
              records[j].data.phoneNumber == records[k].data.phoneNumber ) {
            Ext.Msg.alert("Duplicated Phone Numbers",
              "Interfaces "+ records[j].data.displayName + " and " +
              records[k].data.displayName +
              " have the same phone number (" +
              records[j].data.phoneNumber +
              "). Each Analog FXS interfaces must have a unique phone number."
            );
            return false;
          }
        }
      }
    }
  }
  return true;
}

Paraxip.activateRow = function( sm, index, row) {
  Paraxip.changeRecordActiveState(sm.grid.getStore().getAt(index), true);
}

Paraxip.deactivateRow = function( sm, index, row) {
  Paraxip.changeRecordActiveState(sm.grid.getStore().getAt(index), false);
}

Paraxip.changeRecordActiveState = function(record, newState) {
  record.data.active = newState;
}

Paraxip.selectInterfaces = function( interfacesGridInstance ) {
  var varActiveRecordsCount = 0;
  var bKeepExistingSelection = true;
  var records = interfacesGridInstance.getStore().getRange();
  for( var i = 0; i < records.length; ++i ) {

    if( records[i].data.active ) {
      ++varActiveRecordsCount;
      interfacesGridInstance.getSelectionModel().selectRow(
        i, bKeepExistingSelection);
    }
  }
  // If no records are active it means no records were previously
  // selected, so select them all.
  if (varActiveRecordsCount == 0) {
    interfacesGridInstance.getSelectionModel().selectAll();
  }

  if (records.length > 0 &&
      (varActiveRecordsCount == 0 || records.length == varActiveRecordsCount)) {
    updateHeaderCheckbox(interfacesGridInstance, true);
  }
}

var updateHeaderCheckbox = function(interfacesGridInstance, bSelect) {
    var hd =
      Ext.DomQuery.selectNode(
        '.x-grid3-hd-checker',
        interfacesGridInstance.getSelectionModel().grid.view.innerHd
      );

    if(hd) {
      if (bSelect) {
        Ext.fly(hd).addClass('x-grid3-hd-checker-on');
      } else {
        Ext.fly(hd).removeClass('x-grid3-hd-checker-on');
      }
    }

}

// Changes the headers of an interfaces grid according to the board model
Paraxip.updateInterfacesGridHeaders = function(interfacesGridInstance,record){
  var colModel = interfacesGridInstance.getColumnModel();

  if( Paraxip.isBoardAnalog(record.data.model) ) {
    colModel.setHidden(colModel.getIndexById("id"),false);
    colModel.setHidden(colModel.getIndexById("wanpipe"),true);
    colModel.setHidden(colModel.getIndexById("wanpipeInterface"),false);
    colModel.setHidden(colModel.getIndexById("type"),false);
    colModel.setHidden(colModel.getIndexById("callerName"),false);
    colModel.setHidden(colModel.getIndexById("phoneNumber"),false);
  } else {
    colModel.setHidden(colModel.getIndexById("id"),false);
    colModel.setHidden(colModel.getIndexById("wanpipe"),false);
    colModel.setHidden(colModel.getIndexById("wanpipeInterface"),true);
    colModel.setHidden(colModel.getIndexById("type"),true);
    colModel.setHidden(colModel.getIndexById("callerName"),true);
    colModel.setHidden(colModel.getIndexById("phoneNumber"),true);
  }
}

Paraxip.createAvailableBoardsGrid = function( config, interfacesGridInstance ) {
  var sm = new Ext.grid.ParaxipCheckboxSelectionModel({
    checkOnly: true
    , listeners: {
      "rowselect": function( sm, index, rowData ) {
        Paraxip.activateRow( sm, index, rowData );
      }
      ,"rowdeselect": Paraxip.deactivateRow
    }
  });
  var grid = new Ext.grid.EditorGridPanel(
    Ext.apply({
      layout: 'fit'
      ,height: 350
      ,hideMode: "offsets"
      ,selModel: sm
      ,enableColumnMove: false
      ,enableColumnHide: false
      ,enableColumnResize: false
      ,enableHdMenu: false
      ,columnWidth: .5
      ,columns: [
        sm
        ,{
          header: "ID"
          ,width: 25
          ,sortable: false
          ,dataIndex: 'id'
          ,css: 'padding-top:5px;padding-bottom:5px;'
        }
        ,{
          id: 'displayName'
          ,header: "Name"
          ,width: 125
          ,sortable: false
          ,dataIndex: 'displayName'
          ,css: 'padding-top:5px;padding-bottom:5px;'
          ,editor: new Ext.form.TextField({
            allowBlank: false
          })
        }
        ,{
          header: "Type"
          ,width: 55
          ,sortable: false
          ,dataIndex: 'type'
          ,css: 'padding-top:5px;padding-bottom:5px;'
        }
        ,{
          header: "Model"
          ,width: 75
          ,sortable: false
          ,dataIndex: 'model'
          ,css: 'padding-top:5px;padding-bottom:5px;'
        }
        ,{
          header: "Number of Interfaces"
          ,width: 115
          ,sortable: false
          ,dataIndex: 'numInterfaces'
          ,css: 'padding-top:5px;padding-bottom:5px;'
        }
      ]
      ,stripeRows: true
      ,autoExpandColumn: 'displayName'
      ,title: 'Select Boards'
      ,viewConfig: {forcefit:true}
      ,listeners: {
        "rowclick": function(el, rowIndex, colIndex, eventObj) {
          var record = el.getStore().getAt(rowIndex);

          if (record.data.active) {
            interfacesGridInstance.getStore().load({
              params: {"boardID":record.data.id}
              ,callback: function() {
                Paraxip.selectInterfaces(interfacesGridInstance);
              }
            });
            interfacesGridInstance.setTitle(
              "Select Interfaces for Board " + record.data.id);
          } else {
            interfacesGridInstance.setTitle(
              "Click on a checked board to select its interfaces");
            interfacesGridInstance.getStore().removeAll();
            updateHeaderCheckbox(interfacesGridInstance, false);
          }

          Paraxip.updateInterfacesGridHeaders(interfacesGridInstance, record);

          Paraxip.selectInterfaces(interfacesGridInstance);
        }
        ,"render":function(){
          // Temporary until I find a way to load interfaces with select all
          var index = this.getColumnModel().getIndexById('checker');
          var hd = Ext.fly(this.getView().getHeaderCell(index));
          hd.hide();
        }
      }
    },config
  ));

  return grid;
}

// Specify the store to use in the config object passed to the function
Paraxip.createAvailableInterfacesGrid = function( config ) {
  var sm2 = new Ext.grid.ParaxipCheckboxSelectionModel({
    checkOnly: true
    ,listeners: {
      "rowselect": Paraxip.activateRow
      ,"rowdeselect": function(sm, index, row){
        var result = true;

        // If the beforeDeselectMethod is specified in the config object,
        // call this method before deactivating the row
        if( config.beforeDeselectMethod !== undefined ) {
          result = config.beforeDeselectMethod(sm, index, row);
        }

        // If the beforeDeselectMethod was called and was successfull,
        // deactivate the row. Otherwise, do nothing.
        if( result ) {
          Paraxip.deactivateRow(sm, index, row);
        } else {
          sm.selectRow(index, true /* Keep Existing Selection*/);
        }
      }
    }
  });
  return new Ext.grid.EditorGridPanel(
    Ext.apply({
      layout: 'fit'
      // not brakable spaces tag is required when the rendered title contains
      // spaces to prevent a wrapping in the generated html.
      ,title: '<html>Click&nbsp;on&nbsp;a&nbsp;checked&nbsp;board&nbsp;to&nbsp;select&nbsp;its&nbsp;interfaces</html>'
      ,header: true
      ,height: 350
      ,hideMode: "offsets"
      ,selModel: sm2
      ,enableColumnMove: false
      ,enableColumnHide: false
      ,enableColumnResize: false
      ,enableHdMenu: false
      ,columnWidth: .5
      ,colModel: new Ext.grid.ColumnModel({
        columns: [
                  sm2
                  ,{
                  id: "id"
                      ,header: "ID"
                      ,width: 25
                      ,sortable: false
                      ,dataIndex: 'id'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      }
                  ,{
                  id: 'displayName'
                      ,header: "Name"
                      ,width: 125
                      ,sortable: false
                      ,dataIndex: 'displayName'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      ,editor: new Ext.form.TextField({
                        allowBlank: false
                            })
                      }
                  ,{
                  id: "wanpipe"
                      ,header: "Wanpipe"
                      ,width: 100
                      ,sortable: false
                      ,dataIndex: 'wanpipe'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      }
                  ,{
                  id: "wanpipeInterface"
                      ,header: "Wanpipe Interface"
                      ,width: 100
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'wanpipeInterface'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      }
                  ,{
                  id: "type"
                      ,header: "Type"
                      ,width: 35
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'type'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      }
                  ,{
                  id: "callerName"
                      ,header: "Caller Name"
                      ,width: 95
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'callerName'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      ,editor: new Ext.form.TextField({
                        allowBlank: false
                            })
                      }
                  ,{
                  id: "phoneNumber"
                      ,header: "Phone Number"
                      ,width: 80
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'phoneNumber'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      ,editor: new Ext.form.NumberField({
                        allowBlank: false
                            })
                      },{
                  id: "callerName"
                      ,header: "Caller Name"
                      ,width: 95
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'callerName'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      ,editor: new Ext.form.TextField({
                        allowBlank: false
                            })
                      },{
                  id: "connectionStatus"
                      ,header: "Status"
                      ,width: 95
                      ,sortable: false
                      ,hidden: true
                      ,dataIndex: 'connectionStatus'
                      ,css: 'padding-top:5px;padding-bottom:5px;'
                      ,editor: new Ext.form.TextField({
                        allowBlank: false
                            })
                      }
                  ]

            ,isCellEditable: function(colIndex, rowIndex) {
            var field = this.getDataIndex(colIndex);
            if (field == 'callerName') {
              var store = Ext.getCmp('Paraxip.wizard.availableInterfacesGrid').store;
              var record = store.getAt(rowIndex);
              if(!record){
                store = Ext.getCmp('Paraxip.config.boards.availableInterfacesGrid').store
                record = store.getAt(rowIndex);
              }
              if (record.data.type == "FXO") { //enable_edit is field in record
                return false;//return false to deny editing
              }
            }
            return Ext.grid.ColumnModel.prototype.isCellEditable.call(this, colIndex, rowIndex);
          }
        })
      ,stripeRows: true
      ,autoExpandColumn: 'displayName'
      ,viewConfig: {forcefit:true}
     //  ,listeners: {
//         "beforeedit" : function(e) {
//           // Prevent edition of the caller name cell for FXO interfaces
//           if( e.field == "callerName" ) {
//             if( e.record.data.type == "FXO" ) {
//               return false;
//             }
//           }
//         }
//       }
    },config)
  );
}

function addHelpImage(element, helpWindowName, helpWindowUrl, helpImageLeft) {
  var elp = element.el.findParent('.x-form-element', 5, true);

  var size = elp.getSize();

  element.helpImage = elp.createChild({
    style : "width:16px;height:18px;position:relative;left:" +
            helpImageLeft + "px;" +
            "top:-20px;display:block;background:transparent " +
            "url(/static/images/help.gif) no-repeat 0 2px;" +
            "cursor:pointer!important;"
  });

  elp.setSize(size);

  element.helpImage.on("click", function()
  {
    helpWindow.setTitle(helpWindowName);
    helpWindow.show();

    var textarea = helpWindow.getComponent("helpText_area");
    textarea.body.load({
      url:helpWindowUrl});
  }, element);//scope

}

function isValidCallerName(callerName) {
  var regExpCallerName = /^[\w\-\s]*$/;
  return regExpCallerName.test(callerName);
}

function isValidHostName(hostName) {
  return true;
  var regExpIpV4 = /\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b/;
  var regExpDomain = /\b((?=[a-z0-9-]{1,63}\.)(xn--)?[a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,63}\b/;
  return (hostName == "localhost" ||
          regExpIpV4.test(hostName) ||
          regExpDomain.test(hostName));
}

var boardActive = function (boardsGrid, boardId) {
  var boardActive = false;
  var boards = boardsGrid.getStore().getRange();
  for(var j = 0; j < boards.length && boardActive == false; ++j) {
    if(boards[j].data.id == boardId) {
      boardActive = boards[j].data.active;
    }
  }
  return boardActive;
}
