Ext.namespace("Paraxip.status.logging");

//TODO this datastore should be fetched from the controller via a web service
Ext.data.loglevel = [
  ['FATAL', 'FATAL'],
  ['ERROR', 'ERROR'],
  ['WARN', 'WARN'],
  ['INFO', 'INFO'],
  ['DEBUG', 'DEBUG'],
  ['TRACE', 'TRACE']
];

var storeLogLevelValues = new Ext.data.JsonStore({
  id      : 'storeLogLevelValues',
  url     : Paraxip.baseUrl + 'status/getLogLevel',
  root    : 'data',
  fields  : ['logger','level']
});

var storeGlobalLogConfigProperties = new Ext.data.JsonStore({
  id      : 'storeGlobalLogConfigProperties',
  url     : Paraxip.baseUrl + 'status/getGlobalLogConfigProperties',
  root    : 'data',
  fields  : ['property','value']
});

Ext.data.storeGlobalLogConfigValues = [['PRODUCTION','PRODUCTION'],
                                       ['DEVELOPMENT','DEVELOPMENT']];

Paraxip.status.logging.loadLogLevel = function(){
  Paraxip.status.logging.logLevelGrid.getStore().reload();
};

Paraxip.status.logging.logLevelGrid = new ParaxipEditorGridPanelWithHelp({
  id: 'Paraxip.status.logging.loglevelGrid',
  title: 'Advanced Logger Configuration',
  frame:true,
  cls: 'loglevelGrid',
  autoScroll: true,
  stripeRows: true,
  enableHdMenu: false,
  enableColumnMove: false,
  hideHeaders: false,
  border: true,
  height: 360,
      helpUrl: "status-and-controls/advanced_logger_config_help.html",
  style: 'text-align:left;white-space: normal;',
  selModel    : new Ext.grid.RowSelectionModel(),
  store:  storeLogLevelValues,
  columns: [
    {
      header: "Logger",
      width: 160,
      sortable: true,
      dataIndex: 'logger'
    },
    {
      header: "Level",
      width: 75,
      sortable: true,
      dataIndex: 'level',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : Ext.data.loglevel,
        listClass       : 'x-combo-list-small'
        })
    }
  ],
  viewConfig :{forceFit:true,scrollOffset: 0, autoFill: true},
  buttons:[{
    id: 'Paraxip.status.channels.saveButton',
    text: 'Apply',
    listeners: {
      'click':function(){
        Paraxip.status.logging.saveLogLevel();
      }
    }
    },{
    id: 'Paraxip.status.channels.refreshButton',
    text: 'Refresh',
    listeners: {
      'click':function(){
        Paraxip.status.logging.loadLogLevel();
      }
    }
  }]
});

//------------------------------------------------------------------------------
Paraxip.status.logging.valueComboBox = new Ext.form.ComboBox({
      forceSelection  : true,
      triggerAction   : 'all',
      mode            : 'local',
      selectOnFocus   : true,
      valueField      : 'id',
      displayField    : 'id',
      editable        : false,
      store           : Ext.data.storeGlobalLogConfigValues,
      listClass       : 'x-combo-list-small'
});

Paraxip.status.logging.globalLoggerConfigPanel = new ParaxipEditorGridPanelWithHelp({
  id: 'Paraxip.status.logging.globalLoggerConfigPanel',
  title: 'Basic Logger Configuration',
  frame:true,
  cls: 'loglevelGrid',
  autoScroll: true,
  stripeRows: true,
  enableHdMenu: false,
  enableColumnMove: false,
  hideHeaders: false,
  border: true,
  height: 360,
  helpUrl: "status-and-controls/basic_logger_config_help.html",
  style: 'text-align:left;white-space: normal;',
  selModel    : new Ext.grid.RowSelectionModel(),
  store:   storeGlobalLogConfigProperties,
  columns: [
    {
      header: "Property",
      id: "property",
      width: 160,
      sortable: true,
      frame:true,
      dataIndex: 'property'
    },
    {
      header: "Value",
      id: "value",
      width: 75,
      sortable: true,
      dataIndex: 'value',
      editor: Paraxip.status.logging.valueComboBox
    }
  ],
  viewConfig :{forceFit:true,scrollOffset: 0, autoFill: true},
  buttonAlign : "right",
  buttons:[{
    id: 'Paraxip.status.logging.globalLoggerConfig.applyButton',
    text: 'Apply',
    disabled: true,
    listeners: {
      'click':function(){
        Paraxip.status.logging.globalLoggerConfigPanel.saveGlobalConfiguration();
        this.disable();
      }
    }
  },
  {
    id: 'Paraxip.status.logging.globalLoggerConfig.refreshButton',
    text: 'Refresh',
    listeners: {
      'click':function(){
        Paraxip.status.logging.globalLoggerConfigPanel.getStore().reload();
        Ext.getCmp('Paraxip.status.logging.globalLoggerConfig.applyButton').disable();
      }
    }
  }]
});

Paraxip.status.logging.globalLoggerConfigPanel.on(
 'cellclick',
 function(){

 var rec = Paraxip.status.logging.globalLoggerConfigPanel.getSelectionModel().getSelected();

 if(rec.data.property == 'Mode'){
   Ext.data.storeGlobalLogConfigValues = [['PRODUCTION','PRODUCTION'],
                                        ['DEVELOPMENT','DEVELOPMENT']];
 }else if(rec.data.property == 'SIP Logging' ||
          rec.data.property == 'PSTN Logging' || 
          rec.data.property == 'Media Logging'){                                  
  Ext.data.storeGlobalLogConfigValues = [['NORMAL','NORMAL'],
                                        ['DEBUG','DEBUG']];
 }else if(rec.data.property == 'Root Logger'){
   Ext.data.storeGlobalLogConfigValues = Ext.data.loglevel;
 }else{                                               
  Ext.data.storeGlobalLogConfigValues = [['ENABLED','ENABLED'],
                                        ['DISABLED','DISABLED']];
 }
 Paraxip.status.logging.valueComboBox.store.loadData(Ext.data.storeGlobalLogConfigValues);
});

Paraxip.status.logging.globalLoggerConfigPanel.on(
 'afteredit',
 function(){
 var rec = Paraxip.status.logging.globalLoggerConfigPanel.getStore().getModifiedRecords();
 if(rec){
   Ext.getCmp('Paraxip.status.logging.globalLoggerConfig.applyButton').enable();
 }else{
   Ext.getCmp('Paraxip.status.logging.globalLoggerConfig.applyButton').disable();
 }

});



Paraxip.status.logging.globalLoggerConfigPanel.loadGrid = function(){
  if( Paraxip.status.logging.globalLoggerConfigPanel.rendered ) {
     Paraxip.status.logging.globalLoggerConfigPanel.getStore().load();
  }
};


Paraxip.status.logging.globalLoggerConfigPanel.on(
  'render',
  Paraxip.status.logging.globalLoggerConfigPanel.loadGrid
);



Paraxip.status.logging.globalLoggerConfigPanel.saveGlobalConfiguration = function(){

  var transmitArray = new Array();
  var rec  = Paraxip.status.logging.globalLoggerConfigPanel.getStore().getModifiedRecords();

  for (var i = 0, row; row = rec[i++];) {
    property  = row.data['property'];
    value = row.data['value'];
    var tmpArray = new Array();
    tmpArray.push(property,value);
    transmitArray.push(tmpArray);
  }

  Ext.Ajax.request({
    url: Paraxip.baseUrl + 'status/setGlobalLogConfigProperties',
    method: 'GET',
    params: {
        records:Ext.util.JSON.encode(transmitArray)
    },
    success: function(form, action){
      var responses = Ext.decode(form.responseText);
      Paraxip.status.logging.globalLoggerConfigPanel.getStore().commitChanges();
      Paraxip.status.logging.globalLoggerConfigPanel.getStore().reload();
    }
  });

};


//------------------------------------------------------------------------------


Paraxip.status.logging.loggingPanel = new Ext.Panel({
  title         : "Logger Configuration",
  frame         : true,
  header        : false,
  id        : 'Paraxip.status.logging.loggingPanel',
  layout    : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },

 defaults    : {
    width: 971,
    autoHeight: true,
    fill: true
  },

  width     : 971,
  height    : 486,
  items     : [Paraxip.status.logging.globalLoggerConfigPanel,Paraxip.status.logging.logLevelGrid]
});

Paraxip.status.logging.saveLogLevel = function(){
  var rec = Paraxip.status.logging.logLevelGrid.getStore().getModifiedRecords();
  var transmitArray = new Array();

  // Validate that we have modified records before sending request.
  if (rec.length < 1 ){
    return;
  }

  for (var i = 0, row; row = rec[i++];) {
    level  = row.data['level'];
    logger = row.data['logger'];
    var tmpArray = new Array();
    tmpArray.push(level,logger);
    transmitArray.push(tmpArray);
  }

  Ext.Ajax.request({
    url: Paraxip.baseUrl + 'status/setLogLevel',
    method: 'GET',
    params: {
        records:Ext.util.JSON.encode(transmitArray)
    },
    success: function(form, action){
      var responses = Ext.decode(form.responseText);
      Paraxip.status.logging.logLevelGrid.getStore().commitChanges();
      Paraxip.status.logging.loadLogLevel();
    }
  });

};

Paraxip.status.logging.logLevelGrid.loadGrid = function(){
  if( Paraxip.status.logging.logLevelGrid.rendered ) {
    Paraxip.status.logging.logLevelGrid.getStore().load();
  }
};

Paraxip.status.logging.logLevelGrid.on(
  'render',
  Paraxip.status.logging.logLevelGrid.loadGrid
);


