Ext.namespace("Paraxip.status",
              "Paraxip.status.system",
              "Paraxip.status.controls",
              "Paraxip.status.channels");

// This map contains all the colors associated with the alarms level.
Paraxip.status.system.ErrorColorMap = new Paraxip.Map();
Paraxip.status.system.ErrorColorMap.put('GREEN','#09c300');
Paraxip.status.system.ErrorColorMap.put('YELLOW','#fffd3d');
Paraxip.status.system.ErrorColorMap.put('ORANGE','#ff7a29');
Paraxip.status.system.ErrorColorMap.put('RED','#ff0000');

var systemStatusRefreshTime = 30;

/** 
 * Changes the background color of the highest error value cell
 * by chosing the color value in Paraxip.status.system.ErrorColorMap.
 * @param records: a list of the records received while loading the data.  
 */
Paraxip.status.system.changeErrorBGColor = function(records){
  var cell        = Paraxip.status.system.errorGrid.getView().getCell(0,1);
  var errorValue  = records.data.items[0].data['errorValue'];
  var errorColor  = Paraxip.status.system.ErrorColorMap.get(errorValue);
  cell.setAttribute('bgColor',errorColor);
}

/**
 * Sends a call to the Gateway to clear the last error and bring back the 
 * highest error level to GREEN.
 */
Paraxip.status.system.resetErrors = function(){
  Ext.Ajax.request({
    url: Paraxip.baseUrl + "status/resetErrors",
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(){
      Paraxip.status.system.errorGrid.getStore().reload();          
    }
  });
}

/**
 * Calls the reload method for the 3 grids in the Status&Controls tab
 * to refresh the data.
 */
Paraxip.status.system.reloadPage = function(){
  Paraxip.status.system.errorGrid.getStore().reload();
  Paraxip.status.system.statsGrid.getStore().reload();
}


Paraxip.status.system.errorGrid = new Ext.grid.GridPanel({
  id: 'Paraxip.status.system.errorGrid',
  title: 'Error Report',
  layout: 'fit',
  colspan: 1,
  stripeRows: true,
  hideHeaders: true,
  enableHdMenu: false,
  style: 'white-space: normal;',
  width: 486,
  height: 486,
  border: true,
  store: new Ext.data.JsonStore({
    id: 'errorGridStore',
    url: Paraxip.baseUrl + "status/refreshErrors",
    root: 'data',
    //autoLoad: true,
    fields: [
      {name: 'errorDesc'},
      {name: 'errorValue'}
    ],
    listeners: {
      'load':Paraxip.status.system.changeErrorBGColor
    }
  }),
  columns    :[
    {header: "Error Description", width: 175, sortable: false, dataIndex: 'errorDesc',css:'padding-top:20px;padding-bottom:20px;'},
    {header: "Error Value", width: 175, sortable: false, dataIndex: 'errorValue',align: 'center',css:"padding-top:20px;padding-bottom:20px;"}
  ],
  viewConfig :{forceFit:true, scrollOffset: 0}
});

Paraxip.status.system.errorGrid.on('render',function(){
  this.getStore().load({
    url: Paraxip.baseUrl + "status/refreshErrors"
  });
  this.getStore().startAutoRefresh(systemStatusRefreshTime);

},Paraxip.status.system.errorGrid);

Paraxip.status.system.autoRefreshInterval = new Ext.form.NumberField({
  id: "Paraxip.status.system.autoRefreshInterval"
  ,minValue: 1
  ,value: systemStatusRefreshTime
  ,width: 30
  ,allowNegative: false
  ,allowDecimals: false
  ,allowBlank: false
  ,listeners: {
    'change' : function(e, newVal, oldVal) {
      Paraxip.status.system.statsGrid.getStore().startAutoRefresh(newVal);
      Paraxip.status.system.errorGrid.getStore().startAutoRefresh(newVal);
    }
  }
});

Paraxip.status.system.statsGrid = new Ext.grid.GridPanel({
  title   : 'Call Statistics',
  colspan : 1,
  layout  : 'fit',
  id      : 'Paraxip.status.system.statsGrid',
  width   : 486,
  height  : 486,
  border  : true,
  store   : new Ext.data.JsonStore({
    id: 'statsGridStore',
    url: Paraxip.baseUrl + "status/getStatistics",
    root: 'data',
    fields: [
      {name: 'statName'},
      {name: 'currValue'}
    ]
  }),
  columns:[
    {header: "Statistic Name", width: 185, sortable: false, dataIndex: 'statName',css:'padding-top:4px;padding-bottom:4px;'},
    {header: "Current Value", width: 185, sortable: false, dataIndex: 'currValue',css:'padding-top:4px;padding-bottom:4px;'}
  ],
  viewConfig :{forceFit:true}
});

Paraxip.status.system.statsGrid.on('render',function(){
  this.getStore().load({
    url: Paraxip.baseUrl + "status/getStatistics"
  });
  this.getStore().startAutoRefresh(systemStatusRefreshTime);
},Paraxip.status.system.statsGrid);

Paraxip.status.system.systemPanel = new Ext.Panel({
  id          : 'Paraxip.status.system.systemPanel',
  layout      : 'table',
  width       : 972,
  height      : 486,
  layoutConfig: {
    columns: 2
  },
  items       : [
    Paraxip.status.system.errorGrid,
    Paraxip.status.system.statsGrid
  ],
  buttonAlign : "center",
  bbar: [
  {
    xtype: 'button',
    id: 'Paraxip.status.system.resetButton',
    text: 'Reset Error Report',
    minWidth: 115,
    listeners: {
      'click':Paraxip.status.system.resetErrors
    }
  },
    "->",
  {
    xtype : "label"
    ,text : "Automatic Refresh Interval (s):"
  },
    Paraxip.status.system.autoRefreshInterval,
    "-",
  {
    xtype: 'button',
    id: 'Paraxip.status.system.refreshButton',
    text: 'Refresh Page',
    minWidth: 115,
    listeners: {
      'click':Paraxip.status.system.reloadPage
    }
  }],
  listeners: {
    "render" : function() {
      Ext.Ajax.request({
        url: Paraxip.baseUrl + "getGCParameterValue",
        disableCaching: true,
        params: {"paramName":'netborder.oamWebServer.systemStatusRefreshTimeSec'},
        success: function(result){
          var response = Ext.util.JSON.decode(result.responseText);
          if (response.paramValue) {
            systemStatusRefreshTime = response.paramValue;
            if( Paraxip.status.system.autoRefreshInterval != undefined ) {
              Paraxip.status.system.autoRefreshInterval.setValue(systemStatusRefreshTime);
            }
            if( Paraxip.status.system.statsGrid != undefined ) {
              Paraxip.status.system.statsGrid.getStore().startAutoRefresh(systemStatusRefreshTime);
            }
            if( Paraxip.status.system.errorGrid != undefined ) {
              Paraxip.status.system.errorGrid.getStore().startAutoRefresh(systemStatusRefreshTime);
            }
          }
        }
      });
    }
  }
});

